@@ -8,8 +8,10 @@ from pdm.cli.filters import GroupSelection
 from pdm.cli.hooks import HookManager
 from pdm.cli.options import (
     dry_run_option,
+    frozen_lockfile_option,
     install_group,
     lockfile_option,
+    override_option,
     packages_group,
     prerelease_option,
     save_strategy_group,
@@ -33,7 +35,9 @@ class Command(BaseCommand):
     arguments = (
         *BaseCommand.arguments,
         lockfile_option,
+        frozen_lockfile_option,
         save_strategy_group,
+        override_option,
         update_strategy_group,
         prerelease_option,
         unconstrained_option,
@@ -95,13 +99,13 @@ class Command(BaseCommand):
         no_editable: bool = False,
         no_self: bool = False,
         dry_run: bool = False,
-        prerelease: bool = False,
+        prerelease: bool | None = None,
         fail_fast: bool = False,
         hooks: HookManager | None = None,
     ) -> None:
         """Add packages and install"""
         from pdm.cli.actions import do_lock, do_sync
-        from pdm.cli.utils import check_project_file, populate_requirement_names, save_version_specifiers
+        from pdm.cli.utils import check_project_file, save_version_specifiers
         from pdm.models.requirements import parse_requirement
         from pdm.models.specifiers import get_specifier
         from pdm.utils import normalize_name
@@ -112,49 +116,45 @@ class Command(BaseCommand):
             raise PdmUsageError("Cannot use --no-editable with editable packages given.")
         group = selection.one()
         tracked_names: set[str] = set()
-        requirements: dict[str, Requirement] = {}
+        requirements: list[Requirement] = []
         lock_groups = ["default"] if project.lockfile.empty() else project.lockfile.groups
         if lock_groups is not None and group not in lock_groups:
-            project.core.ui.echo(f"Adding group [success]{group}[/] to lockfile", err=True, style="info")
+            if project.enable_write_lockfile:
+                project.core.ui.info(f"Adding group [success]{group}[/] to lockfile")
             lock_groups.append(group)
-        if (
-            group == "default"
-            or not selection.dev
-            and group not in project.pyproject.settings.get("dev-dependencies", {})
+        if group == "default" or (
+            not selection.dev and normalize_name(group) not in project.pyproject.dev_dependencies
         ):
             if editables:
                 raise PdmUsageError("Cannot add editables to the default or optional dependency group")
         for r in [parse_requirement(line, True) for line in editables] + [parse_requirement(line) for line in packages]:
-            if project.name and normalize_name(project.name) == r.key and not r.extras:
-                project.core.ui.echo(
-                    f"Package [req]{project.name}[/] is the project itself.",
-                    err=True,
-                    style="warning",
-                )
+            if project.is_distribution and normalize_name(name := project.name) == r.key and not r.extras:
+                project.core.ui.warn(f"Package [req]{name}[/] is the project itself.")
                 continue
             if r.is_file_or_url:
                 r.relocate(project.backend)  # type: ignore[attr-defined]
             key = r.identify()
-            r.prerelease = prerelease
             tracked_names.add(key)
-            requirements[key] = r
+            requirements.append(r)
         if requirements:
             project.core.ui.echo(
-                f"Adding packages to [primary]{group}[/] "
+                f"Adding {'[bold]global[/] ' if project.is_global else ''}packages to [primary]{group}[/] "
                 f"{'dev-' if selection.dev else ''}dependencies: "
-                + ", ".join(f"[req]{r.as_line()}[/]" for r in requirements.values())
+                + ", ".join(f"[req]{r.as_line()}[/]" for r in requirements)
             )
+        project.add_dependencies(requirements, group, selection.dev or False, write=False)
         all_dependencies = project.all_dependencies
-        group_deps = all_dependencies.setdefault(group, {})
+        group_deps = all_dependencies[group]
+        for req in group_deps:
+            if req.identify() in tracked_names:
+                req.prerelease = prerelease
         if unconstrained:
             if not requirements:
                 raise PdmUsageError("--unconstrained requires at least one package")
-            for req in group_deps.values():
+            for req in group_deps:
                 req.specifier = get_specifier("")
-        group_deps.update(requirements)
-        reqs = [
-            r for g, deps in all_dependencies.items() if lock_groups is None or g in lock_groups for r in deps.values()
-        ]
+
+        reqs = [r for g, deps in all_dependencies.items() for r in deps if lock_groups is None or g in lock_groups]
         with hooks.skipping("post_lock"):
             resolved = do_lock(
                 project,
@@ -168,19 +168,18 @@ class Command(BaseCommand):
 
         # Update dependency specifiers and lockfile hash.
         deps_to_update = group_deps if unconstrained else requirements
-        save_version_specifiers({group: deps_to_update}, resolved, save)
+        save_version_specifiers(deps_to_update, resolved, save)
         if not dry_run:
             project.add_dependencies(deps_to_update, group, selection.dev or False)
             project.write_lockfile(project.lockfile._data, False)
-            hooks.try_emit("post_lock", resolution=resolved, dry_run=dry_run)
-        populate_requirement_names(group_deps)
+        hooks.try_emit("post_lock", resolution=resolved, dry_run=dry_run)
         if sync:
             do_sync(
                 project,
                 selection=GroupSelection(project, groups=[group], default=False),
                 no_editable=no_editable and tracked_names,
                 no_self=no_self or group != "default",
-                requirements=list(group_deps.values()),
+                requirements=list(group_deps),
                 dry_run=dry_run,
                 fail_fast=fail_fast,
                 hooks=hooks,
