@@ -6,42 +6,23 @@ import json
 import os
 import re
 import sys
-from collections import ChainMap, OrderedDict
-from concurrent.futures import ThreadPoolExecutor
+from collections import OrderedDict
 from fnmatch import fnmatch
+from gettext import gettext as _
 from json import dumps
 from pathlib import Path
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    Iterable,
-    Mapping,
-    MutableMapping,
-    cast,
-    no_type_check,
-)
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Mapping, MutableMapping, cast, no_type_check
 
-import tomlkit
 from packaging.specifiers import SpecifierSet
 from resolvelib.structs import DirectedGraph
 from rich.tree import Tree
 
 from pdm import termui
 from pdm.exceptions import PdmArgumentError, ProjectError
-from pdm.models.requirements import (
-    Requirement,
-    filter_requirements_with_extras,
-    parse_requirement,
-    strip_extras,
-)
+from pdm.models.markers import EnvSpec
+from pdm.models.requirements import Requirement, filter_requirements_with_extras, strip_extras
 from pdm.models.specifiers import PySpecSet, get_specifier
-from pdm.utils import (
-    comparable_version,
-    is_path_relative_to,
-    normalize_name,
-    url_to_path,
-)
+from pdm.utils import comparable_version, is_path_relative_to, normalize_name, url_to_path
 
 if TYPE_CHECKING:
     from argparse import Action, _ArgumentGroup
@@ -52,7 +33,6 @@ if TYPE_CHECKING:
     from pdm.compat import Distribution
     from pdm.compat import importlib_metadata as im
     from pdm.models.candidates import Candidate
-    from pdm.models.repositories import BaseRepository
     from pdm.project import Project
 
 
@@ -85,19 +65,19 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
 
         # no help; start on same line and add a final newline
         if not action.help:
-            tup = self._current_indent, "", action_header
-            action_header = "%*s%s\n" % tup
+            tup = "", self._current_indent, action_header
+            action_header = "{0:>{1}}{2}\n".format(*tup)
 
         # short action name; start on the same line and pad two spaces
         elif len(action_header) <= action_width:
-            tup = self._current_indent, "", action_width, action_header  # type: ignore[assignment]
-            action_header = "%*s%-*s  " % tup  # type: ignore[str-format]
+            tup = "", self._current_indent, action_header, action_width  # type: ignore[assignment]
+            action_header = "{0:>{1}}{2:<{3}}  ".format(*tup)
             indent_first = 0
 
         # long action name; start on the next line
         else:
-            tup = self._current_indent, "", action_header
-            action_header = "%*s%s\n" % tup
+            tup = "", self._current_indent, action_header
+            action_header = "{0:>{1}}{2}\n".format(*tup)
             indent_first = help_position
 
         # Special format for empty action_header
@@ -115,9 +95,9 @@ class PdmFormatter(argparse.RawDescriptionHelpFormatter):
         if action.help:
             help_text = self._expand_help(action)
             help_lines = self._split_lines(help_text, help_width)
-            parts.append("%*s%s\n" % (indent_first, "", help_lines[0]))
+            parts.append("{:>{}}{}\n".format("", indent_first, help_lines[0]))
             for line in help_lines[1:]:
-                parts.append("%*s%s\n" % (help_position, "", line))
+                parts.append("{:>{}}{}\n".format("", help_position, line))
 
         # or add a newline if the description doesn't end with one
         elif not action_header.endswith("\n"):
@@ -150,6 +130,33 @@ class ArgumentParser(argparse.ArgumentParser):
         )
         self._optionals.title = "options"
 
+    def parse_known_args(self, args: Any = None, namespace: Any = None) -> Any:
+        args, argv = super().parse_known_args(args, namespace)
+        if argv:
+            msg = _("unrecognized arguments: %s")
+            self.error(msg % " ".join(argv))
+        return args, argv
+
+
+def format_similar_command(root_command: str, commands: list[str], script_commands: list[str]) -> str:
+    from difflib import get_close_matches
+
+    similar_commands = get_close_matches(root_command, commands)
+    similar_script_commands = get_close_matches(root_command, script_commands)
+    commands_text = "\n".join([f"  - {cmd}" for cmd in similar_commands])
+    script_commands_text = "\n".join([f"  - {cmd}" for cmd in similar_script_commands])
+    message = f"[red]Command not found: {root_command}[/]"
+    if commands_text:
+        message += f"""
+[green]Did you mean one of these commands?
+{commands_text}[/]"""
+
+    if script_commands_text:
+        message += f"""
+[yellow]{"Or" if commands_text else "Did you mean"} one of these script commands?
+{script_commands_text}[/]"""
+    return message
+
 
 class ErrorArgumentParser(ArgumentParser):
     """A subclass of argparse.ArgumentParser that raises
@@ -159,17 +166,17 @@ class ErrorArgumentParser(ArgumentParser):
     """
 
     def _parse_known_args(
-        self, arg_strings: list[str], namespace: argparse.Namespace
+        self, arg_strings: list[str], namespace: argparse.Namespace, *args: Any, **kwargs: Any
     ) -> tuple[argparse.Namespace, list[str]]:
         try:
-            return super()._parse_known_args(arg_strings, namespace)
+            return super()._parse_known_args(arg_strings, namespace, *args, **kwargs)
         except argparse.ArgumentError as e:
             # We raise a dedicated error to avoid being caught by the caller
             raise PdmArgumentError(e) from e
 
 
 @dc.dataclass(frozen=True)
-class Package:
+class PackageNode:
     """An internal class for the convenience of dependency graph building."""
 
     name: str = dc.field(hash=True, compare=True)
@@ -182,37 +189,29 @@ class Package:
 
 def build_dependency_graph(
     working_set: Mapping[str, im.Distribution],
-    marker_env: dict[str, str] | None = None,
+    env_spec: EnvSpec,
     selected: set[str] | None = None,
     include_sub: bool = True,
 ) -> DirectedGraph:
     """Build a dependency graph from locked result."""
-    graph: DirectedGraph[Package | None] = DirectedGraph()
+    graph: DirectedGraph[PackageNode | None] = DirectedGraph()
     graph.add(None)  # sentinel parent of top nodes.
     node_with_extras: set[str] = set()
 
-    def add_package(key: str, dist: Distribution | None) -> Package:
+    def add_package(key: str, dist: Distribution | None) -> PackageNode:
         name, extras = strip_extras(key)
         extras = extras or ()
         reqs: dict[str, Requirement] = {}
         if dist:
-            requirements = (
-                parse_requirement(r)
-                for r in filter_requirements_with_extras(
-                    cast(str, dist.metadata["Name"]),
-                    dist.requires or [],
-                    extras,
-                    include_default=True,
-                )
-            )
+            requirements = filter_requirements_with_extras(dist.requires or [], extras, include_default=True)
             for req in requirements:
-                if not req.marker or req.marker.evaluate(marker_env):
+                if not req.marker or req.marker.matches(env_spec):
                     reqs[req.identify()] = req
             version: str | None = dist.version
         else:
             version = None
 
-        node = Package(key, version, reqs)
+        node = PackageNode(key, version, reqs)
         if node not in graph:
             if extras:
                 node_with_extras.add(name)
@@ -250,13 +249,13 @@ def build_dependency_graph(
 
 
 def specifier_from_requirement(requirement: Requirement) -> str:
-    return str(requirement.specifier or "Any")
+    return str(requirement.specifier) or "Any"
 
 
 def add_package_to_tree(
     root: Tree,
     graph: DirectedGraph,
-    package: Package,
+    package: PackageNode,
     required: list[str],
     visited: frozenset[str] = frozenset(),
 ) -> None:
@@ -292,8 +291,8 @@ def add_package_to_tree(
 def add_package_to_reverse_tree(
     root: Tree,
     graph: DirectedGraph,
-    package: Package,
-    child: Package | None = None,
+    package: PackageNode,
+    child: PackageNode | None = None,
     requires: str = "",
     visited: frozenset[str] = frozenset(),
 ) -> None:
@@ -316,24 +315,24 @@ def add_package_to_reverse_tree(
 
     if package.name in visited:
         return
-    parents: list[Package] = sorted(filter(None, graph.iter_parents(package)), key=lambda p: p.name)
+    parents: list[PackageNode] = sorted(filter(None, graph.iter_parents(package)), key=lambda p: p.name)
     for parent in parents:
         requires = specifier_from_requirement(parent.requirements[package.name])
         add_package_to_reverse_tree(node, graph, parent, package, requires, visited=visited | {package.name})
     return
 
 
-def package_is_project(package: Package, project: Project) -> bool:
-    return project.name is not None and package.name == normalize_name(project.name)
+def package_is_project(package: PackageNode, project: Project) -> bool:
+    return project.is_distribution and package.name == normalize_name(project.name)
 
 
 def _format_forward_dependency_graph(
-    project: Project, graph: DirectedGraph[Package | None], patterns: list[str]
+    project: Project, graph: DirectedGraph[PackageNode | None], patterns: list[str]
 ) -> Tree:
     """Format dependency graph for output."""
     root = Tree("Dependencies", hide_root=True)
 
-    def find_package_to_add(package: Package) -> Package | None:
+    def find_package_to_add(package: PackageNode) -> PackageNode | None:
         if not patterns:
             return package
         to_check = [package]
@@ -344,7 +343,12 @@ def _format_forward_dependency_graph(
             to_check.extend(graph.iter_children(package))
         return None
 
-    all_dependencies = ChainMap(*project.all_dependencies.values())
+    all_dependencies = {
+        r.identify(): r
+        for deps in project.all_dependencies.values()
+        for r in deps
+        if not r.marker or r.marker.matches(project.environment.spec)
+    }
     top_level_dependencies = {find_package_to_add(node) for node in graph.iter_children(None) if node}
     for package in sorted((node for node in top_level_dependencies if node), key=lambda p: p.name):
         required: set[str] = set()
@@ -363,11 +367,11 @@ def _format_forward_dependency_graph(
 
 
 def _format_reverse_dependency_graph(
-    project: Project, graph: DirectedGraph[Package | None], patterns: list[str]
+    project: Project, graph: DirectedGraph[PackageNode | None], patterns: list[str]
 ) -> Tree:
     """Format reverse dependency graph for output."""
 
-    def find_package_to_add(package: Package) -> Package | None:
+    def find_package_to_add(package: PackageNode) -> PackageNode | None:
         if not patterns:
             return package
         to_check = [package]
@@ -388,18 +392,24 @@ def _format_reverse_dependency_graph(
 
 
 def build_forward_dependency_json_subtree(
-    root: Package,
+    root: PackageNode,
     project: Project,
-    graph: DirectedGraph[Package | None],
-    required_by: Package | None = None,
+    graph: DirectedGraph[PackageNode | None],
+    required_by: PackageNode | None = None,
     visited: frozenset[str] = frozenset(),
 ) -> dict:
     required: set[str] = set()
+    all_dependencies = {
+        r.identify(): r
+        for deps in project.all_dependencies.values()
+        for r in deps
+        if not r.marker or r.marker.matches(project.environment.spec)
+    }
     for parent in graph.iter_parents(root):
         if parent:
             r = specifier_from_requirement(parent.requirements[root.name])
         elif not package_is_project(root, project):
-            requirements = required_by.requirements if required_by else ChainMap(*project.all_dependencies.values())
+            requirements = required_by.requirements if required_by else all_dependencies
             if root.name in requirements:
                 r = specifier_from_requirement(requirements[root.name])
             else:
@@ -426,10 +436,10 @@ def build_forward_dependency_json_subtree(
 
 
 def build_reverse_dependency_json_subtree(
-    root: Package,
+    root: PackageNode,
     project: Project,
-    graph: DirectedGraph[Package | None],
-    requires: Package | None = None,
+    graph: DirectedGraph[PackageNode | None],
+    requires: PackageNode | None = None,
     visited: frozenset[str] = frozenset(),
 ) -> dict:
     parents = graph.iter_parents(root) if root.name not in visited else []
@@ -448,14 +458,14 @@ def build_reverse_dependency_json_subtree(
     )
 
 
-def package_match_patterns(package: Package, patterns: list[str]) -> bool:
+def package_match_patterns(package: PackageNode, patterns: list[str]) -> bool:
     return not patterns or any(fnmatch(package.name, pattern) for pattern in patterns)
 
 
 def build_dependency_json_tree(
-    project: Project, graph: DirectedGraph[Package | None], reverse: bool, patterns: list[str]
+    project: Project, graph: DirectedGraph[PackageNode | None], reverse: bool, patterns: list[str]
 ) -> list[dict]:
-    def find_package_to_add(package: Package) -> Package | None:
+    def find_package_to_add(package: PackageNode) -> PackageNode | None:
         if not patterns:
             return package
         to_check = [package]
@@ -469,7 +479,7 @@ def build_dependency_json_tree(
                 to_check.extend(graph.iter_children(package))
         return None
 
-    top_level_packages: Iterable[Package | None]
+    top_level_packages: Iterable[PackageNode | None]
     if reverse:
         top_level_packages = filter(lambda n: not list(graph.iter_children(n)), graph)  # leaf nodes
         build_dependency_json_subtree: Callable = build_reverse_dependency_json_subtree
@@ -485,7 +495,7 @@ def build_dependency_json_tree(
 
 def show_dependency_graph(
     project: Project,
-    graph: DirectedGraph[Package | None],
+    graph: DirectedGraph[PackageNode | None],
     reverse: bool = False,
     json: bool = False,
     patterns: list[str] | None = None,
@@ -509,85 +519,43 @@ def show_dependency_graph(
     echo(tree)
 
 
-def format_lockfile(
-    project: Project,
-    mapping: dict[str, Candidate],
-    fetched_dependencies: dict[tuple[str, str | None], list[Requirement]],
-    groups: list[str] | None = None,
-    cross_platform: bool | None = None,
-    static_urls: bool | None = None,
-) -> dict:
-    """Format lock file from a dict of resolved candidates, a mapping of dependencies
-    and a collection of package summaries.
-    """
-    from pdm.formats.base import make_array, make_inline_table
-
-    packages = tomlkit.aot()
-    for _k, v in sorted(mapping.items()):
-        base = tomlkit.table()
-        base.update(v.as_lockfile_entry(project.root))
-        base.add("summary", v.summary or "")
-        deps = make_array(sorted(r.as_line() for r in fetched_dependencies[v.dep_key]), True)
-        if len(deps) > 0:
-            base.add("dependencies", deps)
-        if v.hashes:
-            collected = {}
-            for item in v.hashes:
-                if static_urls:
-                    row = {"url": item["url"], "hash": item["hash"]}
-                else:
-                    row = {"file": item["file"], "hash": item["hash"]}
-                inline = make_inline_table(row)
-                # deduplicate and sort
-                collected[tuple(row.values())] = inline
-            if collected:
-                base.add("files", make_array([collected[k] for k in sorted(collected)], True))
-        packages.append(base)
-    doc = tomlkit.document()
-    metadata = tomlkit.table()
-    if groups is None:
-        groups = list(project.iter_groups())
-    metadata.update(
-        {
-            "groups": sorted(groups, key=lambda k: k != "default"),
-            "cross_platform": cross_platform if cross_platform is not None else project.lockfile.cross_platform,
-            "static_urls": static_urls if static_urls is not None else project.lockfile.static_urls,
-        }
-    )
-    doc.add("metadata", metadata)
-    doc.add("package", packages)
-    return cast(dict, doc)
-
-
 def save_version_specifiers(
-    requirements: dict[str, dict[str, Requirement]],
-    resolved: dict[str, Candidate],
+    requirements: Iterable[Requirement],
+    resolved: dict[str, list[Candidate]],
     save_strategy: str,
 ) -> None:
     """Rewrite the version specifiers according to the resolved result and save strategy
 
     :param requirements: the requirements to be updated
     :param resolved: the resolved mapping
-    :param save_strategy: compatible/wildcard/exact
+    :param save_strategy: compatible/safe_compatible/wildcard/exact
     """
 
-    def candidate_version(c: Candidate) -> Version:
+    def candidate_version(candidates: list[Candidate]) -> Version | None:
+        if len(candidates) > 1:
+            return None
+        c = candidates[0]
         assert c.version is not None
         return comparable_version(c.version)
 
-    for reqs in requirements.values():
-        for name, r in reqs.items():
-            if r.is_named and not r.specifier:
-                if save_strategy == "exact":
-                    r.specifier = get_specifier(f"=={candidate_version(resolved[name])}")
-                elif save_strategy == "compatible":
-                    version = candidate_version(resolved[name])
-                    if version.is_prerelease or version.is_devrelease:
-                        r.specifier = get_specifier(f">={version},<{version.major + 1}")
-                    else:
+    for r in requirements:
+        name = r.identify()
+        if r.is_named and not r.specifier and name in resolved:
+            version = candidate_version(resolved[name])
+            if version is None:
+                continue
+            if save_strategy == "exact":
+                r.specifier = get_specifier(f"=={version}")
+            elif save_strategy in ["compatible", "safe_compatible"]:
+                if version.is_prerelease or version.is_devrelease:
+                    r.specifier = get_specifier(f">={version},<{version.major + 1}")
+                else:
+                    if save_strategy == "compatible":
                         r.specifier = get_specifier(f"~={version.major}.{version.minor}")
-                elif save_strategy == "minimum":
-                    r.specifier = get_specifier(f">={candidate_version(resolved[name])}")
+                    else:
+                        r.specifier = get_specifier(f"~={version}")
+            elif save_strategy == "minimum":
+                r.specifier = get_specifier(f">={version}")
 
 
 def check_project_file(project: Project) -> None:
@@ -638,6 +606,8 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
 
     causes: list[RequirementInformation] = err.causes
     info_lines: set[str] = set()
+    if not causes:
+        return ""
     if all(isinstance(cause.requirement, PythonRequirement) for cause in causes):
         project_requires: PythonRequirement = next(cause.requirement for cause in causes if cause.parent is None)
         pyspec = cast(PySpecSet, project_requires.specifier)
@@ -655,7 +625,7 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
             pyspec &= req.specifier
             info_lines.add(f"  {req.as_line()} (from {parent!r})")
         result.extend(sorted(info_lines))
-        if pyspec.is_impossible:
+        if pyspec.is_empty():
             result.append("Consider changing the version specifiers of the dependencies to be compatible")
         else:
             result.append(
@@ -681,7 +651,7 @@ def format_resolution_impossible(err: ResolutionImpossible) -> str:
     result.extend(sorted(info_lines))
     result.append(
         "To fix this, you could loosen the dependency version constraints in "
-        "pyproject.toml. See https://pdm.fming.dev/latest/usage/dependency/"
+        "pyproject.toml. See https://pdm-project.org/en/latest/usage/lockfile/"
         "#solve-the-locking-failure for more details."
     )
     return "\n".join(result)
@@ -705,16 +675,6 @@ def merge_dictionary(target: MutableMapping[Any, Any], input: Mapping[Any, Any],
             target[key] = value
 
 
-def fetch_hashes(repository: BaseRepository, mapping: Mapping[str, Candidate]) -> None:
-    """Fetch hashes for candidates in parallel"""
-
-    def do_fetch(candidate: Candidate) -> None:
-        candidate.hashes = repository.get_hashes(candidate)
-
-    with ThreadPoolExecutor() as executor:
-        executor.map(do_fetch, mapping.values())
-
-
 def is_pipx_installation() -> bool:
     return sys.prefix.split(os.sep)[-3:-1] == ["pipx", "venvs"]
 
@@ -761,15 +721,10 @@ def use_venv(project: Project, name: str) -> None:
     from pdm.environments import PythonEnvironment
 
     venv = get_venv_with_name(project, cast(str, name))
-    project.core.ui.echo(
-        f"In virtual environment: [success]{venv.root}[/]", err=True, style="info", verbosity=termui.Verbosity.DETAIL
-    )
+    project.core.ui.info(f"In virtual environment: [success]{venv.root}[/]", verbosity=termui.Verbosity.DETAIL)
     project.environment = PythonEnvironment(project, python=str(venv.interpreter))
 
 
-def populate_requirement_names(req_mapping: dict[str, Requirement]) -> None:
-    # Update the requirement key if the name changed.
-    for key, req in list(req_mapping.items()):
-        if key and key.startswith(":empty:"):
-            req_mapping[req.identify()] = req
-            del req_mapping[key]
+def normalize_pattern(pattern: str) -> str:
+    """Normalize a pattern to a valid name for a package."""
+    return re.sub(r"[^A-Za-z0-9*?]+", "-", pattern).lower()
