@@ -5,36 +5,62 @@ r"""
  / ____/ /_/ / /  / /
 /_/   /_____/_/  /_/
 """
+
 from __future__ import annotations
 
 import argparse
+import contextlib
+import dataclasses as dc
 import importlib
 import itertools
 import os
 import pkgutil
 import sys
+from datetime import datetime
+from functools import cached_property
 from pathlib import Path
-from typing import Any, Iterable, cast
+from tempfile import TemporaryDirectory
+from typing import TYPE_CHECKING, ClassVar, cast
 
-from resolvelib import Resolver
+import tomlkit.exceptions
 
 from pdm import termui
 from pdm.__version__ import __version__
-from pdm.cli.actions import check_update
-from pdm.cli.commands.base import BaseCommand
-from pdm.cli.options import ignore_python_option, pep582_option, verbose_option
-from pdm.cli.utils import ErrorArgumentParser, PdmFormatter
+from pdm.cli.options import ignore_python_option, no_cache_option, non_interactive_option, pep582_option, verbose_option
+from pdm.cli.utils import ArgumentParser, ErrorArgumentParser, format_similar_command
 from pdm.compat import importlib_metadata
 from pdm.exceptions import PdmArgumentError, PdmUsageError
-from pdm.installers import InstallManager, Synchronizer
-from pdm.models.repositories import PyPIRepository
+from pdm.installers import InstallManager
+from pdm.models.repositories import BaseRepository, PyPIRepository
 from pdm.project import Project
-from pdm.project.config import Config, ConfigItem
+from pdm.project.config import Config
 from pdm.utils import is_in_zipapp
 
+if TYPE_CHECKING:
+    from typing import Any, Iterable
+
+    from pdm.cli.commands.base import BaseCommand
+    from pdm.project.config import ConfigItem
+
 COMMANDS_MODULE_PATH = importlib.import_module("pdm.cli.commands").__path__
 
 
+@dc.dataclass
+class State:
+    """State of the core object."""
+
+    config_settings: dict[str, Any] | None = None
+    """The config settings map shared by all packages"""
+    exclude_newer: datetime | None = None
+    """The exclude newer than datetime for the lockfile"""
+    build_isolation: bool = True
+    """Whether to make an isolated environment and install requirements for build"""
+    enable_cache: bool = True
+    """Whether to enable the cache"""
+    overrides: list[str] = dc.field(default_factory=list)
+    """The requirement overrides for the resolver"""
+
+
 class Core:
     """A high level object that manages all classes and configurations"""
 
@@ -42,24 +68,27 @@ class Core:
     subparsers: argparse._SubParsersAction
 
     project_class = Project
-    repository_class = PyPIRepository
-    resolver_class = Resolver
-    synchronizer_class = Synchronizer
+    repository_class: type[BaseRepository] = PyPIRepository
     install_manager_class = InstallManager
+    commands: ClassVar[list[str]] = []
 
     def __init__(self) -> None:
         self.version = __version__
-        self.ui = termui.UI()
+        self.exit_stack = contextlib.ExitStack()
+        self.ui = termui.UI(exit_stack=self.exit_stack)
+        self.state = State()
+        self.exit_stack.callback(setattr, self, "config_settings", None)
         self.init_parser()
         self.load_plugins()
 
+    def create_temp_dir(self, *args: Any, **kwargs: Any) -> str:
+        return self.exit_stack.enter_context(TemporaryDirectory(*args, **kwargs))
+
     def init_parser(self) -> None:
         self.parser = ErrorArgumentParser(
             prog="pdm",
             description=__doc__,
-            formatter_class=PdmFormatter,
         )
-        self.parser.is_root = True  # type: ignore[attr-defined]
         self.parser.add_argument(
             "-V",
             "--version",
@@ -68,19 +97,20 @@ class Core:
                 termui.style("PDM", style="bold"),
                 termui.style(self.version, style="success"),
             ),
-            help="show the version and exit",
+            help="Show the version and exit",
         )
         self.parser.add_argument(
             "-c",
             "--config",
-            help="Specify another config file path(env var: PDM_CONFIG_FILE)",
+            help="Specify another config file path [env var: PDM_CONFIG_FILE] ",
         )
-        self.parser._positionals.title = "Commands"
         verbose_option.add_to_parser(self.parser)
+        no_cache_option.add_to_parser(self.parser)
         ignore_python_option.add_to_parser(self.parser)
         pep582_option.add_to_parser(self.parser)
+        non_interactive_option.add_to_parser(self.parser)
 
-        self.subparsers = self.parser.add_subparsers(parser_class=argparse.ArgumentParser)
+        self.subparsers = self.parser.add_subparsers(parser_class=ArgumentParser, title="commands", metavar="")
         for _, name, _ in pkgutil.iter_modules(COMMANDS_MODULE_PATH):
             module = importlib.import_module(f"pdm.cli.commands.{name}", __name__)
             try:
@@ -104,6 +134,7 @@ class Core:
                 is_global=global_project,
                 global_config=options.config or os.getenv("PDM_CONFIG_FILE"),
             )
+        self.state.build_isolation = project.config["build_isolation"]
         return project
 
     def create_project(
@@ -132,8 +163,14 @@ class Core:
 
         self.ui.set_verbosity(options.verbose)
         self.ui.set_theme(project.global_config.load_theme())
+        self.ui.log_dir = os.path.expanduser(cast(str, project.config["log_dir"]))
 
         command = cast("BaseCommand | None", getattr(options, "command", None))
+        self.state.config_settings = getattr(options, "config_setting", None)
+
+        if options.no_cache:
+            self.state.enable_cache = False
+
         hooks = HookManager(project, getattr(options, "skip", None))
         hooks.try_emit("pre_invoke", command=command.name if command else None, options=options)
 
@@ -143,16 +180,54 @@ class Core:
         for callback in getattr(options, "callbacks", []):
             callback(project, options)
 
-        if getattr(options, "lockfile", None):
-            project.set_lockfile(cast(str, options.lockfile))
+        if lockfile := getattr(options, "lockfile", None):
+            project.set_lockfile(cast(str, lockfile))
 
         if getattr(options, "use_venv", None):
             use_venv(project, cast(str, options.use_venv))
 
+        if overrides := getattr(options, "override", None):
+            self.state.overrides = overrides
+
         if command is None:
-            self.parser.error("No command given")
+            self.parser.print_help()
+            sys.exit(0)
         command.handle(project, options)
 
+    @staticmethod
+    def get_command(args: list[str]) -> tuple[int, str]:
+        """Get the command name from the arguments"""
+        options_with_values = ("-c", "--config")
+        need_value = False
+        for i, arg in enumerate(args):
+            if arg.startswith("-"):
+                if not arg.startswith(options_with_values):
+                    continue
+                if (arg.startswith("-c") and arg != "-c") or arg.startswith("--config="):
+                    continue
+                need_value = True
+            elif need_value:
+                need_value = False
+                continue
+            else:
+                return i, arg
+        return -1, ""
+
+    def _get_cli_args(self, args: list[str], obj: Project | None) -> list[str]:
+        project = self.create_project(is_global=False) if obj is None else obj
+        if project.is_global:
+            return args
+        try:
+            config = project.pyproject.settings.get("options", {})
+        except tomlkit.exceptions.TOMLKitError as e:  # pragma: no cover
+            self.ui.error(f"Failed to parse pyproject.toml: {e}")
+            config = {}
+        (pos, command) = self.get_command(args)
+        if command and command in config:
+            # add args after the command
+            args[pos + 1 : pos + 1] = list(config[command])
+        return args
+
     def main(
         self,
         args: list[str] | None = None,
@@ -161,9 +236,9 @@ class Core:
         **extra: Any,
     ) -> None:
         """The main entry function"""
-
-        # Ensure same behavior while testing and using the CLI
-        args = args or sys.argv[1:]
+        if args is None:
+            args = []
+        args = self._get_cli_args(args, obj)
         # Keep it for after project parsing to check if its a defined script
         root_script = None
         try:
@@ -172,7 +247,7 @@ class Core:
             # Failed to parse, try to give all to `run` command as shortcut
             # and keep to root script (first non-dashed param) to check existence
             # as soon as the project is parsed
-            root_script = next((arg for arg in args if not arg.startswith("-")), None)
+            _, root_script = self.get_command(args)
             if not root_script:
                 self.parser.error(str(e.__cause__))
             try:
@@ -182,7 +257,9 @@ class Core:
 
         project = self.ensure_project(options, obj)
         if root_script and root_script not in project.scripts:
-            self.parser.error(f"Script unknown: {root_script}")
+            message = format_similar_command(root_script, self.commands, list(project.scripts.keys()))
+            message = termui.style(message)
+            self.parser.error(message)
 
         try:
             self.handle(project, options)
@@ -196,10 +273,12 @@ class Core:
                 err=True,
             )
             if should_show_tb:
-                self.ui.echo("Add '-v' to see the detailed traceback", style="warning", err=True)
+                self.ui.warn("Add '-v' to see the detailed traceback", verbosity=termui.Verbosity.NORMAL)
             sys.exit(1)
         else:
             if project.config["check_update"] and not is_in_zipapp():
+                from pdm.cli.actions import check_update
+
                 check_update(project)
 
     def register_command(self, command: type[BaseCommand], name: str | None = None) -> None:
@@ -213,6 +292,8 @@ class Core:
                 is used
         """
         assert self.subparsers
+        if name:
+            self.commands.append(name)
         command.register_to(self.subparsers, name)
 
     @staticmethod
@@ -225,6 +306,31 @@ class Core:
         """
         Config.add_config(name, config_item)
 
+    def _add_project_plugins_library(self) -> None:
+        project = self.create_project(is_global=False)
+        if project.is_global or not project.root.joinpath(".pdm-plugins").exists():
+            return
+
+        import site
+        import sysconfig
+
+        base = str(project.root / ".pdm-plugins")
+        replace_vars = {"base": base, "platbase": base}
+
+        scheme_names = sysconfig.get_scheme_names()
+        if (sys.platform == "darwin" and "osx_framework_library" in scheme_names) or sys.platform == "linux":
+            scheme = "posix_prefix"
+        # sysconfig._get_default_scheme is a private function in 3.8 & 3.9
+        elif sys.version_info < (3, 10):
+            scheme = "nt" if os.name == "nt" else "posix_prefix"
+        else:
+            scheme = sysconfig.get_default_scheme()
+        purelib = sysconfig.get_path("purelib", scheme, replace_vars)
+        scripts = sysconfig.get_path("scripts", scheme, replace_vars)
+        site.addsitedir(purelib)
+        if os.path.exists(scripts):
+            os.environ["PATH"] = os.pathsep.join([scripts, os.getenv("PATH", "")])
+
     def load_plugins(self) -> None:
         """Import and load plugins under `pdm.plugin` namespace
         A plugin is a callable that accepts the core object as the only argument.
@@ -235,6 +341,7 @@ class Core:
                 ...
             ```
         """
+        self._add_project_plugins_library()
         entry_points: Iterable[importlib_metadata.EntryPoint] = itertools.chain(
             importlib_metadata.entry_points(group="pdm"),
             importlib_metadata.entry_points(group="pdm.plugin"),
@@ -243,13 +350,43 @@ class Core:
             try:
                 plugin.load()(self)
             except Exception as e:
-                self.ui.echo(
+                self.ui.error(
                     f"Failed to load plugin {plugin.name}={plugin.value}: {e}",
-                    style="error",
-                    err=True,
                 )
 
+    @cached_property
+    def uv_cmd(self) -> list[str]:
+        from pdm.compat import importlib_metadata
+
+        self.ui.info("Using uv is experimental and might break due to uv updates.")
+        # First, try to find uv in Python modules
+        try:
+            importlib_metadata.distribution("uv")
+        except ModuleNotFoundError:
+            pass
+        else:
+            return [sys.executable, "-m", "uv"]
+        # Try to find it in the typical place:
+        for bin_dir in [".local/bin", ".cargo/bin"]:
+            if (uv_path := Path.home() / bin_dir / "uv").exists():
+                return [str(uv_path)]
+        # If not found, try to find it in PATH
+        import shutil
+
+        path = shutil.which("uv")
+        if path:
+            return [path]
+        # If not found, try to find in the bin dir:
+        if (uv_path := Path(sys.argv[0]).with_name("uv")).exists():
+            return [str(uv_path)]
+        raise PdmUsageError(
+            "use_uv is enabled but can't find uv, please install it first: "
+            "https://docs.astral.sh/uv/getting-started/installation/"
+        )
+
 
 def main(args: list[str] | None = None) -> None:
     """The CLI entry function"""
-    return Core().main(args)
+    core = Core()
+    with core.exit_stack:
+        return core.main(args or sys.argv[1:])
