@@ -11,21 +11,21 @@ from pdm.compat import tomllib
 from pdm.formats.base import (
     MetaConverter,
     Unset,
+    array_of_inline_tables,
     convert_from,
     make_array,
     make_inline_table,
-    parse_name_email,
 )
-from pdm.models.markers import Marker
-from pdm.models.requirements import Requirement
+from pdm.models.markers import Marker, get_marker
+from pdm.models.requirements import FileRequirement, Requirement
 from pdm.models.specifiers import PySpecSet
+from pdm.utils import cd
 
 if TYPE_CHECKING:
-    from pdm.project.core import Project
     from argparse import Namespace
-    from pdm._types import RequirementDict, Source
 
-from pdm.utils import cd
+    from pdm._types import RequirementDict
+    from pdm.project import Project
 
 
 def check_fingerprint(project: Project | None, filename: Path | str) -> bool:
@@ -45,17 +45,17 @@ VERSION_RE = re.compile(r"([^\d\s]*)\s*(\d.*?)\s*(?=,|$)")
 
 def _convert_specifier(version: str) -> str:
     parts = []
-    for op, version in VERSION_RE.findall(str(version)):
+    for op, ver in VERSION_RE.findall(str(version)):
         if op == "~":
             op += "="
         elif op == "^":
-            major, *vparts = version.split(".")
+            major, *vparts = ver.split(".")
             next_major = ".".join([str(int(major) + 1)] + ["0"] * len(vparts))
-            parts.append(f">={version},<{next_major}")
+            parts.append(f">={ver},<{next_major}")
             continue
         elif not op:
             op = "=="
-        parts.append(f"{op}{version}")
+        parts.append(f"{op}{ver}")
     return ",".join(parts)
 
 
@@ -66,14 +66,22 @@ def _convert_python(python: str) -> PySpecSet:
     return functools.reduce(operator.or_, parts)
 
 
-def _convert_req(
-    name: str, req_dict: RequirementDict | list[RequirementDict]
-) -> Iterable[str]:
+def _convert_req(name: str, req_dict: RequirementDict | list[RequirementDict]) -> Iterable[str]:
+    from pdm.models.backends import DEFAULT_BACKEND
+
+    backend = DEFAULT_BACKEND(Path.cwd())
+
+    def fix_req_path(req: Requirement) -> Requirement:
+        if isinstance(req, FileRequirement):
+            req.relocate(backend)
+        return req
+
     if isinstance(req_dict, list):
         for req in req_dict:
             yield from _convert_req(name, req)
     elif isinstance(req_dict, str):
-        yield Requirement.from_req_dict(name, _convert_specifier(req_dict)).as_line()
+        pdm_req = fix_req_path(Requirement.from_req_dict(name, _convert_specifier(req_dict)))
+        yield pdm_req.as_line()
     else:
         assert isinstance(req_dict, dict)
         req_dict = dict(req_dict)
@@ -82,20 +90,39 @@ def _convert_req(
             req_dict["version"] = _convert_specifier(str(req_dict["version"]))
         markers: list[Marker] = []
         if "markers" in req_dict:
-            markers.append(Marker(req_dict.pop("markers")))  # type: ignore
+            markers.append(get_marker(req_dict.pop("markers")))  # type: ignore[arg-type]
         if "python" in req_dict:
-            markers.append(
-                Marker(_convert_python(str(req_dict.pop("python"))).as_marker_string())
-            )
+            markers.append(get_marker(_convert_python(str(req_dict.pop("python"))).as_marker_string()))
         if markers:
-            req_dict["marker"] = str(functools.reduce(operator.and_, markers)).replace(
-                '"', "'"
-            )
+            req_dict["marker"] = str(functools.reduce(operator.and_, markers)).replace('"', "'")
         if "rev" in req_dict or "branch" in req_dict or "tag" in req_dict:
             req_dict["ref"] = req_dict.pop(
-                "rev", req_dict.pop("tag", req_dict.pop("branch", None))  # type: ignore
+                "rev",
+                req_dict.pop("tag", req_dict.pop("branch", None)),  # type: ignore[arg-type]
             )
-        yield Requirement.from_req_dict(name, req_dict).as_line()
+        pdm_req = fix_req_path(Requirement.from_req_dict(name, req_dict))
+        yield pdm_req.as_line()
+
+
+# See https://github.com/python-poetry/poetry-core/pull/521#issuecomment-1327689551
+# for reasoning why email.utils.parseaddr is not used here.
+NAME_EMAIL_RE = re.compile(
+    r"^(?P<name>[- .,\w'’\"():&]+)(?: <(?P<email>.+?)>)?$",  # noqa: RUF001
+    re.UNICODE,
+)
+
+
+def parse_name_email(name_email: list[str]) -> list[str]:
+    return array_of_inline_tables(
+        [
+            {
+                k: v
+                for k, v in NAME_EMAIL_RE.match(item).groupdict().items()  # type: ignore[union-attr]
+                if v is not None
+            }
+            for item in name_email
+        ]
+    )
 
 
 class PoetryMetaConverter(MetaConverter):
@@ -128,9 +155,7 @@ class PoetryMetaConverter(MetaConverter):
         return rv
 
     @convert_from("plugins", name="entry-points")
-    def entry_points(
-        self, value: dict[str, dict[str, str]]
-    ) -> dict[str, dict[str, str]]:
+    def entry_points(self, value: dict[str, dict[str, str]]) -> dict[str, dict[str, str]]:
         return value
 
     @convert_from()
@@ -138,16 +163,12 @@ class PoetryMetaConverter(MetaConverter):
         rv = []
         value, extras = dict(source["dependencies"]), source.pop("extras", {})
         for key, req_dict in value.items():
-            optional = getattr(req_dict, "items", None) and req_dict.pop(
-                "optional", False
-            )
+            optional = getattr(req_dict, "items", None) and req_dict.pop("optional", False)
             for req in _convert_req(key, req_dict):
                 if optional:
                     extra = next((k for k, v in extras.items() if key in v), None)
                     if extra:
-                        self._data.setdefault("optional-dependencies", {}).setdefault(
-                            extra, []
-                        ).append(req)
+                        self._data.setdefault("optional-dependencies", {}).setdefault(extra, []).append(req)
                 else:
                     rv.append(req)
         del source["dependencies"]
@@ -155,24 +176,42 @@ class PoetryMetaConverter(MetaConverter):
 
     @convert_from("dev-dependencies")
     def dev_dependencies(self, value: dict) -> None:
-        self.settings["dev-dependencies"] = {
-            "dev": make_array(
-                [r for key, req in value.items() for r in _convert_req(key, req)], True
-            ),
-        }
+        self.settings.setdefault("dev-dependencies", {})["dev"] = make_array(
+            [r for key, req in value.items() for r in _convert_req(key, req)], True
+        )
+        raise Unset()
+
+    @convert_from("group")
+    def group_dependencies(self, value: dict[str, dict[str, Any]]) -> None:
+        for name, group in value.items():
+            self.settings.setdefault("dev-dependencies", {})[name] = make_array(
+                [r for key, req in group.get("dependencies", {}).items() for r in _convert_req(key, req)], True
+            )
+        raise Unset()
+
+    @convert_from("package-mode")
+    def package_mode(self, value: bool) -> None:
+        self.settings["distribution"] = value
         raise Unset()
 
     @convert_from()
     def includes(self, source: dict[str, list[str] | str]) -> list[str]:
-        result: list[str] = []
+        includes: list[str] = []
+        source_includes: list[str] = []
         for item in source.pop("packages", []):
             assert isinstance(item, dict)
             include = item["include"]
             if item.get("from"):
-                include = f"{item.get('from')}/{include}"
-            result.append(include)
-        result.extend(source.pop("include", []))
-        self.settings.setdefault("build", {})["includes"] = result
+                include = Path(str(item.get("from")), include).as_posix()
+            includes.append(include)
+        for item in source.pop("include", []):
+            if not isinstance(item, dict):
+                includes.append(item)
+            else:
+                dest = source_includes if "sdist" in item.get("format", "") else includes
+                dest.append(item["path"])
+        if includes:
+            self.settings.setdefault("build", {})["includes"] = includes
         raise Unset()
 
     @convert_from("exclude")
@@ -182,18 +221,14 @@ class PoetryMetaConverter(MetaConverter):
 
     @convert_from("build")
     def build(self, value: str | dict) -> None:
-        run_setuptools = True
-        if isinstance(value, dict):
-            if "generate-setup-file" in value:
-                run_setuptools = cast(bool, value["generate-setup-file"])
-            value = value["script"]
-        self.settings.setdefault("build", {}).update(
-            {"setup-script": value, "run-setuptools": run_setuptools}
-        )
+        result = {}
+        if isinstance(value, dict) and "generate-setup-file" in value:
+            result["run-setuptools"] = cast(bool, value["generate-setup-file"])
+        self.settings.setdefault("build", {}).update(result)
         raise Unset()
 
     @convert_from("source")
-    def sources(self, value: list[Source]) -> None:
+    def sources(self, value: list[dict[str, Any]]) -> None:
         self.settings["source"] = [
             {
                 "name": item.get("name", ""),
@@ -211,9 +246,7 @@ def convert(
     options: Namespace | None,
 ) -> tuple[Mapping[str, Any], Mapping[str, Any]]:
     with open(filename, "rb") as fp, cd(os.path.dirname(os.path.abspath(filename))):
-        converter = PoetryMetaConverter(
-            tomllib.load(fp)["tool"]["poetry"], project.core.ui if project else None
-        )
+        converter = PoetryMetaConverter(tomllib.load(fp)["tool"]["poetry"], project.core.ui if project else None)
         return converter.convert()
 
 
