@@ -3,8 +3,9 @@ from __future__ import annotations
 import collections
 import dataclasses
 import os
+from functools import cached_property
 from pathlib import Path
-from typing import Any, Callable, Iterator, Mapping, MutableMapping
+from typing import Any, Callable, ClassVar, Iterator, Mapping, MutableMapping, cast
 
 import platformdirs
 import rich.theme
@@ -15,6 +16,7 @@ from pdm._types import RepositoryConfig
 from pdm.exceptions import NoConfigError, PdmUsageError
 
 REPOSITORY = "repository"
+SOURCE = "pypi"
 DEFAULT_REPOSITORIES = {
     "pypi": "https://upload.pypi.org/legacy/",
     "testpypi": "https://test.pypi.org/legacy/",
@@ -32,9 +34,9 @@ def load_config(file_path: Path) -> dict[str, Any]:
     def get_item(sub_data: Mapping[str, Any]) -> dict[str, Any]:
         result: dict[str, Any] = {}
         for k, v in sub_data.items():
-            if k == "pypi":
+            if k in (REPOSITORY, SOURCE):
                 result.update((f"{k}.{sub_k}", sub_v) for sub_k, sub_v in v.items())
-            elif k != REPOSITORY and isinstance(v, Mapping):
+            elif isinstance(v, Mapping):
                 result.update({f"{k}.{sub_k}": sub_v for sub_k, sub_v in get_item(v).items()})
             else:
                 result.update({k: v})
@@ -53,6 +55,13 @@ def ensure_boolean(val: Any) -> bool:
     return bool(val) and val.lower() not in ("false", "no", "0")
 
 
+def split_by_comma(val: list[str] | str) -> list[str]:
+    """Split a string value by comma"""
+    if isinstance(val, str):
+        return [v.strip() for v in val.split(",")]
+    return val
+
+
 DEFAULT_PYPI_INDEX = "https://pypi.org/simple"
 
 
@@ -85,11 +94,18 @@ class ConfigItem:
 class Config(MutableMapping[str, str]):
     """A dict-like object for configuration key and values"""
 
-    _config_map: dict[str, ConfigItem] = {
+    _config_map: ClassVar[dict[str, ConfigItem]] = {
         "cache_dir": ConfigItem(
             "The root directory of cached files",
             platformdirs.user_cache_dir("pdm"),
             True,
+            env_var="PDM_CACHE_DIR",
+        ),
+        "log_dir": ConfigItem(
+            "The root directory of log files",
+            platformdirs.user_log_dir("pdm"),
+            True,
+            env_var="PDM_LOG_DIR",
         ),
         "check_update": ConfigItem(
             "Check if there is any newer version available",
@@ -105,6 +121,16 @@ class Config(MutableMapping[str, str]):
             "PDM_BUILD_ISOLATION",
             ensure_boolean,
         ),
+        "request_timeout": ConfigItem(
+            "The timeout for network requests in seconds", 15, True, "PDM_REQUEST_TIMEOUT", coerce=int
+        ),
+        "use_uv": ConfigItem(
+            "Use uv for faster resolution and installation",
+            False,
+            False,
+            "PDM_USE_UV",
+            ensure_boolean,
+        ),
         "global_project.fallback": ConfigItem(
             "Use the global project implicitly if no local project is found",
             False,
@@ -123,13 +149,6 @@ class Config(MutableMapping[str, str]):
             True,
         ),
         "global_project.user_site": ConfigItem("Whether to install to user site", False, True, coerce=ensure_boolean),
-        "project_max_depth": ConfigItem(
-            "The max depth to search for a project through the parents",
-            5,
-            True,
-            env_var="PDM_PROJECT_MAX_DEPTH",
-            coerce=int,
-        ),
         "strategy.update": ConfigItem("The default strategy for updating packages", "reuse", False),
         "strategy.save": ConfigItem("Specify how to save versions when a package is added", "minimum", False),
         "strategy.resolve_max_rounds": ConfigItem(
@@ -138,6 +157,9 @@ class Config(MutableMapping[str, str]):
             env_var="PDM_RESOLVE_MAX_ROUNDS",
             coerce=int,
         ),
+        "strategy.inherit_metadata": ConfigItem(
+            "Inherit the groups and markers from parents for each package", True, coerce=ensure_boolean
+        ),
         "install.parallel": ConfigItem(
             "Whether to perform installation and uninstallation in parallel",
             True,
@@ -150,26 +172,39 @@ class Config(MutableMapping[str, str]):
             coerce=ensure_boolean,
         ),
         "install.cache_method": ConfigItem(
-            "`symlink` or `pth` to create links to the cached installation",
+            "Specify how to create links to the caches(`symlink/hardlink`)",
             "symlink",
         ),
+        "python.providers": ConfigItem(
+            "List of python provider names for findpython", default=[], coerce=split_by_comma
+        ),
         "python.use_pyenv": ConfigItem("Use the pyenv interpreter", True, coerce=ensure_boolean),
         "python.use_venv": ConfigItem(
             "Use virtual environments when available", True, env_var="PDM_USE_VENV", coerce=ensure_boolean
         ),
+        "python.install_root": ConfigItem(
+            "The root directory to install python interpreters",
+            global_only=True,
+            default=os.path.join(platformdirs.user_data_dir("pdm"), "python"),
+        ),
         "pypi.url": ConfigItem(
             "The URL of PyPI mirror, defaults to https://pypi.org/simple",
             DEFAULT_PYPI_INDEX,
             env_var="PDM_PYPI_URL",
         ),
-        "pypi.verify_ssl": ConfigItem("Verify SSL certificate when query PyPI", True, coerce=ensure_boolean),
+        "pypi.verify_ssl": ConfigItem(
+            "Verify SSL certificate when query PyPI",
+            True,
+            env_var="PDM_PYPI_VERIFY_SSL",
+            coerce=ensure_boolean,
+        ),
         "pypi.username": ConfigItem("The username to access PyPI", env_var="PDM_PYPI_USERNAME"),
         "pypi.password": ConfigItem("The password to access PyPI", env_var="PDM_PYPI_PASSWORD"),
         "pypi.ca_certs": ConfigItem(
             "Path to a CA certificate bundle used for verifying the identity of the PyPI server", global_only=True
         ),
         "pypi.ignore_stored_index": ConfigItem(
-            "Ignore the configured indexes",
+            "Don't add the indexes from the config that is not listed in project",
             False,
             env_var="PDM_IGNORE_STORED_INDEX",
             coerce=ensure_boolean,
@@ -182,6 +217,12 @@ class Config(MutableMapping[str, str]):
             env_var="PDM_PYPI_JSON_API",
             coerce=ensure_boolean,
         ),
+        "scripts.show_header": ConfigItem(
+            "Display script name and help before running",
+            default=False,
+            env_var="PDM_SCRIPTS_SHOW_HEADER",
+            coerce=ensure_boolean,
+        ),
         "venv.location": ConfigItem(
             "Parent directory for virtualenvs",
             os.path.join(platformdirs.user_data_dir("pdm"), "venvs"),
@@ -224,9 +265,13 @@ class Config(MutableMapping[str, str]):
         defaults = {k: v.default for k, v in cls._config_map.items() if v.should_show()}
         if cls.site is None:
             cls.site = Config(platformdirs.site_config_path("pdm") / "config.toml")
-        defaults.update(cls.site)
+        defaults.update(cls.site.self_data)
         return defaults
 
+    @cached_property
+    def env_map(self) -> Mapping[str, Any]:
+        return EnvMap(self._config_map)
+
     @classmethod
     def add_config(cls, name: str, item: ConfigItem) -> None:
         """Add or modify a config item"""
@@ -237,7 +282,11 @@ class Config(MutableMapping[str, str]):
         self.config_file = config_file.resolve()
         self.deprecated = {v.replace: k for k, v in self._config_map.items() if v.replace}
         self._file_data = load_config(self.config_file)
-        self._data = collections.ChainMap(self._file_data, self.get_defaults() if is_global else {})
+        self._data = collections.ChainMap(
+            cast(MutableMapping[str, Any], self.env_map) if not is_global else {},
+            self._file_data,
+            self.get_defaults() if is_global else {},
+        )
 
     def load_theme(self) -> rich.theme.Theme:
         if not self.is_global:  # pragma: no cover
@@ -248,10 +297,10 @@ class Config(MutableMapping[str, str]):
     def self_data(self) -> dict[str, Any]:
         return dict(self._file_data)
 
-    def iter_sources(self) -> Iterator[tuple[str, RepositoryConfig]]:
+    def iter_sources(self) -> Iterator[RepositoryConfig]:
         for name, data in self._data.items():
-            if name.startswith("pypi.") and name not in self._config_map:
-                yield name[5:], RepositoryConfig(**data)
+            if name.startswith(f"{SOURCE}.") and name not in self._config_map and data:
+                yield RepositoryConfig(**data, name=name[len(SOURCE) + 1 :], config_prefix=SOURCE)
 
     def _save_config(self) -> None:
         """Save the changed to config file."""
@@ -271,59 +320,53 @@ class Config(MutableMapping[str, str]):
 
     def __getitem__(self, key: str) -> Any:
         parts = key.split(".")
-        if parts[0] == REPOSITORY:
+        if parts[0] in (REPOSITORY, SOURCE) and key not in self._config_map:
             if len(parts) < 2:
-                raise PdmUsageError("Must specify a repository name")
-            repo = self.get_repository_config(parts[1])
+                raise PdmUsageError(f"Must specify a {parts[0]} name")
+            repo = self.get_repository_config(parts[1], parts[0])
             if repo is None:
-                raise KeyError(f"No repository named {parts[1]}")
-
-            value = getattr(repo, parts[2]) if len(parts) >= 3 else repo
-            if len(parts) >= 3 and parts[2] == "password" and value:
-                return "<hidden>"
-            return value
-        elif parts[0] == "pypi" and key not in self._config_map:
-            index_key = ".".join(parts[:2])
-            if index_key not in self._data:
-                raise KeyError(f"No PyPI index named {parts[1]}")
-            source = self._data[index_key]
-            if len(parts) >= 3 and parts[2] == "password":
-                return "<hidden>"
-            return source[parts[2]] if len(parts) >= 3 else RepositoryConfig(**self._data[index_key])
-        elif key == "pypi.password":
-            return "<hidden>"
+                raise KeyError(f"No {parts[0]} named {parts[1]}")
+            if len(parts) >= 3:
+                repo.populate_keyring_auth()
+                return getattr(repo, parts[2])
+            return repo
 
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
         config_key = self.deprecated.get(key, key)
         config = self._config_map[config_key]
-        env_var = config.env_var
-        if env_var is not None and env_var in os.environ:
-            result = os.environ[env_var]
+
+        if config_key in self._data:
+            result = self._data[config_key]
+        elif config.replace:
+            result = self._data[config.replace]
         else:
-            if config_key in self._data:
-                result = self._data[config_key]
-            elif config.replace:
-                result = self._data[config.replace]
-            else:
-                raise NoConfigError(key) from None
+            raise NoConfigError(key) from None
         return config.coerce(result)
 
     def __setitem__(self, key: str, value: Any) -> None:
+        from pdm.models.auth import keyring
+
         parts = key.split(".")
-        if parts[0] == REPOSITORY:
+        if parts[0] in (REPOSITORY, SOURCE) and key not in self._config_map:
             if len(parts) < 3:
-                raise PdmUsageError("Set repository config with [success]repository.{name}.{attr}")
-            self._file_data.setdefault(parts[0], {}).setdefault(parts[1], {}).setdefault(parts[2], value)
-            self._save_config()
-            return
-        if parts[0] == "pypi" and key not in self._config_map:
-            if len(parts) < 3:
-                raise PdmUsageError("Set index config with [success]pypi.{name}.{attr}")
+                raise PdmUsageError(f"Set {parts[0]} config with [success]{parts[0]}.{{name}}.{{attr}}")
             index_key = ".".join(parts[:2])
+            username = self._data.get(index_key, {}).get("username")  # type: ignore[call-overload]
+            service = f"pdm-{index_key.replace('.', '-')}"
+            if (
+                parts[2] == "password"
+                and self.is_global
+                and username
+                and keyring.save_auth_info(service, username, value)
+            ):
+                return
+            if parts[2] == "verify_ssl":
+                value = ensure_boolean(value)
             self._file_data.setdefault(index_key, {})[parts[2]] = value
             self._save_config()
             return
+
         if key not in self._config_map and key not in self.deprecated:
             raise NoConfigError(key)
         config_key = self.deprecated.get(key, key)
@@ -332,12 +375,8 @@ class Config(MutableMapping[str, str]):
             raise ValueError(f"Config item '{key}' is not allowed to set in project config.")
 
         value = config.coerce(value)
-        env_var = config.env_var
-        if env_var is not None and env_var in os.environ:
-            ui.echo(
-                "WARNING: the config is shadowed by env var '{}', the value set won't take effect.".format(env_var),
-                style="warning",
-            )
+        if key in self.env_map:
+            ui.warn(f"the config is shadowed by env var '{config.env_var}', the value set won't take effect.")
         self._file_data[config_key] = value
         if config.replace:
             self._file_data.pop(config.replace, None)
@@ -350,32 +389,32 @@ class Config(MutableMapping[str, str]):
         keys: set[str] = set()
         for key in self._data:
             if key in self.deprecated:
-                keys.add(self.deprecated[key])
-            elif key != REPOSITORY:
-                keys.add(key)
+                key = self.deprecated[key]
+            keys.add(key)
         return iter(keys)
 
     def __delitem__(self, key: str) -> None:
+        from pdm.models.auth import keyring
+
         parts = key.split(".")
-        if parts[0] == REPOSITORY:
+        if parts[0] in (REPOSITORY, SOURCE) and key not in self._config_map:
             if len(parts) < 2:
-                raise PdmUsageError("Should specify the name of repository")
-            if len(parts) >= 3:
-                del self._file_data.get(REPOSITORY, {}).get(parts[1], {})[parts[2]]
-            else:
-                del self._file_data.get(REPOSITORY, {})[parts[1]]
-            self._save_config()
-            return
-        if parts[0] == "pypi" and key not in self._config_map:
-            if len(parts) < 2:
-                raise PdmUsageError("Should specify the name of index")
+                raise PdmUsageError(f"Should specify the name of {parts[0]}")
+            index_key = ".".join(parts[:2])
+            username = self._data.get(index_key, {}).get("username")  # type: ignore[call-overload]
+            service = f"pdm-{index_key.replace('.', '-')}"
             if len(parts) >= 3:
                 index_key, attr = key.rsplit(".", 1)
-                del self._file_data.get(index_key, {})[attr]
+                if attr == "password" and username:
+                    keyring.delete_auth_info(service, username)
+                self._file_data.get(index_key, {}).pop(attr, None)
             else:
                 del self._file_data[key]
+                if username:
+                    keyring.delete_auth_info(service, username)
             self._save_config()
             return
+
         config_key = self.deprecated.get(key, key)
         config = self._config_map[config_key]
         self._file_data.pop(config_key, None)
@@ -384,34 +423,67 @@ class Config(MutableMapping[str, str]):
 
         env_var = config.env_var
         if env_var is not None and env_var in os.environ:
-            ui.echo(
-                "WARNING: the config is shadowed by env var '{}', set value won't take effect.".format(env_var),
-                style="warning",
-            )
+            ui.warn(f"The config is shadowed by env var '{env_var}', set value won't take effect.")
         self._save_config()
 
-    def get_repository_config(self, name_or_url: str) -> RepositoryConfig | None:
-        """Get a repository by name or url."""
-        if not self.is_global:  # pragma: no cover
-            raise NoConfigError("repository")
-        repositories: Mapping[str, RepositoryConfig] = {
-            k: RepositoryConfig(**v) for k, v in self._data.get(REPOSITORY, {}).items()
-        }
+    def get_repository_config(self, name_or_url: str, prefix: str) -> RepositoryConfig | None:
+        """Get a repository or source by name or url."""
+        repositories: dict[str, RepositoryConfig] = {}
+        for k, v in self._data.items():
+            if not k.startswith(f"{prefix}.") or k in self._config_map:
+                continue
+            key = k[len(prefix) + 1 :]
+            repositories[key] = RepositoryConfig(**v, name=key, config_prefix=prefix)
         config: RepositoryConfig | None = None
         if "://" in name_or_url:
             config = next(
                 (v for v in repositories.values() if v.url == name_or_url),
-                RepositoryConfig(name_or_url),
+                RepositoryConfig(url=name_or_url, name="__unknown__", config_prefix=prefix),
             )
         else:
             config = repositories.get(name_or_url)
 
+        if prefix == SOURCE or not self.is_global:
+            return config
+
         if name_or_url in DEFAULT_REPOSITORIES:
             if config is None:
-                return RepositoryConfig(DEFAULT_REPOSITORIES[name_or_url])
+                return RepositoryConfig(url=DEFAULT_REPOSITORIES[name_or_url], name=name_or_url, config_prefix=prefix)
             config.passive_update(url=DEFAULT_REPOSITORIES[name_or_url])
         if name_or_url in DEFAULT_REPOSITORIES.values():
+            name = next(k for k, v in DEFAULT_REPOSITORIES.items() if v == name_or_url)
             if config is None:
-                return RepositoryConfig(name_or_url)
+                return RepositoryConfig(
+                    name=name,
+                    config_prefix=prefix,
+                    url=name_or_url,
+                )
             config.passive_update(url=name_or_url)
+            if config.name == "__unknown__":
+                config.name = name
         return config
+
+
+class EnvMap(Mapping[str, Any]):
+    def __init__(self, config_items: Mapping[str, ConfigItem]) -> None:
+        self._config_map = config_items
+
+    def __repr__(self) -> str:
+        return repr(dict(self))
+
+    def __getitem__(self, k: str) -> Any:
+        try:
+            item = self._config_map[k]
+            if item.env_var:
+                return item.coerce(os.environ[item.env_var])
+        except KeyError:
+            pass
+        raise KeyError(k)
+
+    def __iter__(self) -> Iterator[str]:
+        for key, item in self._config_map.items():
+            if item.env_var and item.env_var in os.environ:
+                yield key
+
+    def __len__(self) -> int:
+        return sum(1 for _ in self)
