@@ -1,7 +1,7 @@
 import pytest
 from unearth import Link
 
-from pdm.installers.packages import CachedPackage
+from pdm.models.cached_package import CachedPackage
 from tests import FIXTURES
 
 
@@ -146,8 +146,7 @@ def test_cache_info(project, pdm):
             "http://fixtures.test/artifacts/demo-0.0.1.tar.gz#sha384=9130e5e4912bc78b"
             "1ffabbf406d56bc74b9165b0adc8c627168b7b563b80d5ff6c30e269398d01144ee52aa3"
             "3292682d",
-            "sha384:9130e5e4912bc78b1ffabbf406d56bc74b9165b0adc8c627168b7b563b80d5ff6"
-            "c30e269398d01144ee52aa33292682d",
+            "sha384:9130e5e4912bc78b1ffabbf406d56bc74b9165b0adc8c627168b7b563b80d5ff6c30e269398d01144ee52aa33292682d",
         ),
         (
             "http://fixtures.test/artifacts/demo-0.0.1.tar.gz#md5=5218509812c9fcb4646adde8fd3307e1",
@@ -156,19 +155,17 @@ def test_cache_info(project, pdm):
     ],
 )
 def test_hash_cache(project, url, hash):
-    with project.environment.get_finder() as finder:
-        hash_cache = project.make_hash_cache()
-        assert hash_cache.get_hash(Link(url), finder.session) == hash
+    hash_cache = project.make_hash_cache()
+    assert hash_cache.get_hash(Link(url), project.environment.session) == hash
 
 
 def test_clear_package_cache(project, pdm):
-    pkg = CachedPackage(project.cache("packages") / "test_package")
-    pkg.path.mkdir()
+    pkg = CachedPackage(project.cache("packages") / "test_package.whl.cache")
+    pkg.path.mkdir(parents=True)
     refer_pkg = project.root / "refer_pkg"
     refer_pkg.mkdir()
     pkg.add_referrer(str(refer_pkg))
     assert len(pkg.referrers) == 1
-    pkg._referrers = None
 
     refer_pkg.rmdir()
     pdm(["cache", "clear", "packages"], obj=project, strict=True)
