@@ -34,7 +34,7 @@ def test_list_dependency_graph(project, pdm):
     pdm(["add", "requests"], obj=project, strict=True)
     result = pdm(["list", "--tree"], obj=project)
     expected = "-- urllib3 1.22 [ required: <1.24,>=1.21.1 ]" in result.outputs
-    assert expected
+    assert expected, result.outputs
 
 
 @mock.patch("rich.console.ConsoleOptions.ascii_only", lambda: True)
@@ -399,7 +399,7 @@ def test_list_freeze_banned_options(project, pdm):
     expected = "--resolve cannot be used with --freeze"
     assert expected in result.outputs
 
-    result = pdm(["list", "--freeze", "--sort", "name"], obj=project)
+    result = pdm(["list", "--freeze", "--sort", "version"], obj=project)
     expected = "--sort cannot be used with --freeze"
     assert expected in result.outputs
 
@@ -513,8 +513,16 @@ def fake_working_set(working_set):
         def read_text(self, *args, **kwargs):
             return self.license_text
 
-    # Foo package.
-    foo = Distribution("foo", "0.1.0", metadata={"License": "A License"})
+    # Foo package (adapted to contain newlines in License field)
+    # e.g. via license = { file="LICENSE" }
+    foo = Distribution(
+        "foo",
+        "0.1.0",
+        metadata={
+            "License": "A License\n\nextra\ntext",
+            "Classifier": "License :: A License",
+        },
+    )
     foo_l = _MockPackagePath("foo-0.1.0.dist-info", "LICENSE")
     foo_l.license_text = "license text for foo here"
     foo.files = [foo_l]
@@ -562,8 +570,9 @@ def fake_working_set(working_set):
 def fake_metadata(mocker, repository):
     def prepare_metadata(self):
         can = self.candidate
-        dist = Distribution(can.name, can.version, can.req.editable)
-        dist.dependencies = repository.get_raw_dependencies(can)
+        version, dependencies = repository.get_raw_dependencies(can)
+        dist = Distribution(can.name, version, can.req.editable)
+        dist.dependencies = dependencies
         return dist
 
     return mocker.patch(
@@ -577,14 +586,7 @@ def fake_metadata(mocker, repository):
 def test_list_freeze(project, pdm):
     pdm(["add", "requests"], obj=project, strict=True)
     result = pdm(["list", "--freeze"], obj=project)
-    expected = (
-        "certifi==2018.11.17\n"
-        "chardet==3.0.4\n"
-        "idna==2.7\n"
-        "requests==2.19.1\n"
-        "test-project==0.0.0\n"
-        "urllib3==1.22\n"
-    )
+    expected = "certifi==2018.11.17\nchardet==3.0.4\nidna==2.7\nrequests==2.19.1\ntest-project==0.0.0\nurllib3==1.22\n"
     assert expected == result.output
 
 
@@ -638,11 +640,11 @@ def test_list_bare_fields_licences(project, pdm):
         "+---------------------------------------------------------+\n"
         "| name       | version | groups | licenses                |\n"
         "|------------+---------+--------+-------------------------|\n"
-        "| foo        | 0.1.0   | :sub   | A License               |\n"
         "| bar        | 3.0.1   | :sub   | B License               |\n"
         "| baz        | 2.7     | :sub   | C License               |\n"
-        "| unknown    | 1.0     | :sub   | Apache Software License |\n"
         "| classifier | 1.0     | :sub   | PDM TEST D              |\n"
+        "| foo        | 0.1.0   | :sub   | A License               |\n"
+        "| unknown    | 1.0     | :sub   | Apache Software License |\n"
         "+---------------------------------------------------------+\n"
     )
     assert expected == result.output
@@ -653,11 +655,11 @@ def test_list_csv_fields_licences(project, pdm):
     result = pdm(["list", "--csv", "--fields", "name,version,licenses"], obj=project)
     expected = (
         "name,version,licenses\n"
-        "foo,0.1.0,A License\n"
         "bar,3.0.1,B License\n"
         "baz,2.7,C License\n"
-        "unknown,1.0,Apache Software License\n"
         "classifier,1.0,PDM TEST D\n"
+        "foo,0.1.0,A License\n"
+        "unknown,1.0,Apache Software License\n"
     )
     assert expected == result.output
 
@@ -666,11 +668,11 @@ def test_list_csv_fields_licences(project, pdm):
 def test_list_json_fields_licences(project, pdm):
     result = pdm(["list", "--json", "--fields", "name,version,licenses"], obj=project)
     expected = [
-        {"name": "foo", "version": "0.1.0", "licenses": "A License"},
         {"name": "bar", "version": "3.0.1", "licenses": "B License"},
         {"name": "baz", "version": "2.7", "licenses": "C License"},
-        {"name": "unknown", "version": "1.0", "licenses": "Apache Software License"},
         {"name": "classifier", "version": "1.0", "licenses": "PDM TEST D"},
+        {"name": "foo", "version": "0.1.0", "licenses": "A License"},
+        {"name": "unknown", "version": "1.0", "licenses": "Apache Software License"},
     ]
 
     assert expected == json.loads(result.outputs)
@@ -678,18 +680,12 @@ def test_list_json_fields_licences(project, pdm):
 
 @pytest.mark.usefixtures("fake_working_set")
 def test_list_markdown_fields_licences(project, pdm):
+    # Note that in "foo" the "License" metadata field ("License": "A License\n\nextra\ntext")
+    # is ignored, in favour of the classifier and the LICENSE file.
+    # This behaviour could be improved.
     result = pdm(["list", "--markdown", "--fields", "name,version,licenses"], obj=project)
     expected = (
         "# test-project licenses\n"
-        "## foo\n\n"
-        "| Name | foo |\n"
-        "|----|----|\n"
-        "| Version | 0.1.0 |\n"
-        "| Licenses | A License |\n\n"
-        "foo-0.1.0.dist-info/LICENSE\n\n\n"
-        "````\n"
-        "license text for foo here\n"
-        "````\n\n\n"
         "## bar\n\n"
         "| Name | bar |\n"
         "|----|----|\n"
@@ -708,15 +704,6 @@ def test_list_markdown_fields_licences(project, pdm):
         "````\n"
         "license text for baz here\n"
         "````\n\n\n"
-        "## unknown\n\n"
-        "| Name | unknown |\n"
-        "|----|----|\n"
-        "| Version | 1.0 |\n"
-        "| Licenses | Apache Software License |\n\n"
-        "unknown-1.0.dist-info/COPYING\n\n\n"
-        "````\n"
-        "license text for UNKNOWN here\n"
-        "````\n\n\n"
         "## classifier\n\n"
         "| Name | classifier |\n"
         "|----|----|\n"
@@ -726,6 +713,24 @@ def test_list_markdown_fields_licences(project, pdm):
         "````\n"
         "Problem finding license text: division by zero\n"
         "````\n\n\n"
+        "## foo\n\n"
+        "| Name | foo |\n"
+        "|----|----|\n"
+        "| Version | 0.1.0 |\n"
+        "| Licenses | A License |\n\n"
+        "foo-0.1.0.dist-info/LICENSE\n\n\n"
+        "````\n"
+        "license text for foo here\n"
+        "````\n\n\n"
+        "## unknown\n\n"
+        "| Name | unknown |\n"
+        "|----|----|\n"
+        "| Version | 1.0 |\n"
+        "| Licenses | Apache Software License |\n\n"
+        "unknown-1.0.dist-info/COPYING\n\n\n"
+        "````\n"
+        "license text for UNKNOWN here\n"
+        "````\n\n\n"
     )
     assert expected == result.output
 
