@@ -1,7 +1,5 @@
 import pytest
 
-from pdm.models.requirements import parse_requirement
-
 
 @pytest.mark.usefixtures("working_set")
 def test_update_packages_with_top(project, pdm):
@@ -25,11 +23,19 @@ def test_update_ignore_constraints(project, repository, pdm):
 
     pdm(["update", "pytz"], obj=project, strict=True)
     assert project.pyproject.metadata["dependencies"] == ["pytz~=2019.3"]
-    assert project.locked_repository.all_candidates["pytz"].version == "2019.3"
+    assert project.get_locked_repository().candidates["pytz"].version == "2019.3"
 
     pdm(["update", "pytz", "--unconstrained"], obj=project, strict=True)
     assert project.pyproject.metadata["dependencies"] == ["pytz~=2020.2"]
-    assert project.locked_repository.all_candidates["pytz"].version == "2020.2"
+    assert project.get_locked_repository().candidates["pytz"].version == "2020.2"
+
+    pdm(["add", "chardet"], obj=project, strict=True)
+    assert "chardet~=3.0" in project.pyproject.metadata["dependencies"]
+    assert project.get_locked_repository().candidates["chardet"].version == "3.0.4"
+    repository.add_candidate("chardet", "3.0.6")
+
+    pdm(["update", "chardet", "--unconstrained", "--save-safe-compatible"], obj=project, strict=True)
+    assert "chardet~=3.0.6" in project.pyproject.metadata["dependencies"]
 
 
 @pytest.mark.usefixtures("working_set")
@@ -50,7 +56,7 @@ def test_update_all_packages(project, repository, pdm, strategy):
         ],
     )
     result = pdm(["update", f"--update-{strategy}"], obj=project, strict=True)
-    locked_candidates = project.locked_repository.all_candidates
+    locked_candidates = project.get_locked_repository().candidates
     assert locked_candidates["requests"].version == "2.20.0"
     assert locked_candidates["chardet"].version == ("3.0.5" if strategy == "all" else "3.0.4")
     assert locked_candidates["pytz"].version == "2019.6"
@@ -67,7 +73,7 @@ def test_update_no_lock(project, working_set, repository, pdm):
     pdm(["update", "--frozen-lockfile"], obj=project, strict=True)
     assert working_set["pytz"].version == "2019.6"
     project.lockfile.reload()
-    assert project.locked_repository.all_candidates["pytz"].version == "2019.3"
+    assert project.get_locked_repository().candidates["pytz"].version == "2019.3"
 
 
 @pytest.mark.usefixtures("working_set")
@@ -88,7 +94,7 @@ def test_update_dry_run(project, repository, pdm):
     )
     result = pdm(["update", "--dry-run"], obj=project, strict=True)
     project.lockfile.reload()
-    locked_candidates = project.locked_repository.all_candidates
+    locked_candidates = project.get_locked_repository().candidates
     assert locked_candidates["requests"].version == "2.19.1"
     assert locked_candidates["chardet"].version == "3.0.4"
     assert locked_candidates["pytz"].version == "2019.3"
@@ -133,7 +139,7 @@ def test_update_specified_packages(project, repository, pdm):
         ],
     )
     pdm(["update", "requests"], obj=project, strict=True)
-    locked_candidates = project.locked_repository.all_candidates
+    locked_candidates = project.get_locked_repository().candidates
     assert locked_candidates["requests"].version == "2.20.0"
     assert locked_candidates["chardet"].version == "3.0.4"
 
@@ -155,12 +161,74 @@ def test_update_specified_packages_eager_mode(project, repository, pdm):
         ],
     )
     pdm(["update", "requests", "--update-eager"], obj=project, strict=True)
-    locked_candidates = project.locked_repository.all_candidates
+    locked_candidates = project.get_locked_repository().candidates
     assert locked_candidates["requests"].version == "2.20.0"
     assert locked_candidates["chardet"].version == "3.0.5"
     assert locked_candidates["pytz"].version == "2019.3"
 
 
+@pytest.mark.usefixtures("working_set")
+def test_update_transitive(project, repository, pdm):
+    pdm(["add", "requests", "--no-sync"], obj=project, strict=True)
+    repository.add_candidate("chardet", "3.0.5")
+    repository.add_candidate("requests", "2.20.0")
+    repository.add_dependencies(
+        "requests",
+        "2.20.0",
+        [
+            "certifi>=2017.4.17",
+            "chardet<3.1.0,>=3.0.2",
+            "idna<2.8,>=2.5",
+            "urllib3<1.24,>=1.21.1",
+        ],
+    )
+    pdm(["update", "chardet"], obj=project, strict=True)
+    locked_candidates = project.get_locked_repository().candidates
+    assert not any("chardet" in dependency for dependency in project.pyproject.metadata["dependencies"])
+    assert locked_candidates["chardet"].version == "3.0.5"
+    assert locked_candidates["requests"].version == "2.19.1"
+
+
+@pytest.mark.usefixtures("working_set")
+def test_update_transitive_nonexistent_dependencies(project, pdm):
+    pdm(["add", "requests", "--no-sync"], obj=project, strict=True)
+    result = pdm(["update", "numpy"], obj=project)
+    assert "ProjectError" in result.stderr
+    assert "numpy does not exist in" in result.stderr
+
+
+@pytest.mark.usefixtures("working_set")
+def test_update_package_wrong_group(project, pdm):
+    pdm(["add", "-d", "requests"], obj=project, strict=True)
+    result = pdm(["update", "requests"], obj=project)
+    assert "ProjectError" in result.stderr
+    assert "requests does not exist in default, but exists in dev" in result.stderr
+
+
+@pytest.mark.usefixtures("working_set")
+def test_update_transitive_non_transitive_dependencies(project, repository, pdm):
+    pdm(["add", "requests", "pytz", "--no-sync"], obj=project, strict=True)
+    repository.add_candidate("pytz", "2019.6")
+    repository.add_candidate("chardet", "3.0.5")
+    repository.add_candidate("requests", "2.20.0")
+    repository.add_dependencies(
+        "requests",
+        "2.20.0",
+        [
+            "certifi>=2017.4.17",
+            "chardet<3.1.0,>=3.0.2",
+            "idna<2.8,>=2.5",
+            "urllib3<1.24,>=1.21.1",
+        ],
+    )
+    pdm(["update", "requests", "chardet", "pytz"], obj=project, strict=True)
+    locked_candidates = project.get_locked_repository().candidates
+    assert not any("chardet" in dependency for dependency in project.pyproject.metadata["dependencies"])
+    assert locked_candidates["requests"].version == "2.20.0"
+    assert locked_candidates["chardet"].version == "3.0.5"
+    assert locked_candidates["pytz"].version == "2019.6"
+
+
 @pytest.mark.usefixtures("working_set")
 def test_update_specified_packages_eager_mode_config(project, repository, pdm):
     pdm(["add", "requests", "pytz", "--no-sync"], obj=project, strict=True)
@@ -179,7 +247,7 @@ def test_update_specified_packages_eager_mode_config(project, repository, pdm):
     )
     pdm(["config", "strategy.update", "eager"], obj=project, strict=True)
     pdm(["update", "requests"], obj=project, strict=True)
-    locked_candidates = project.locked_repository.all_candidates
+    locked_candidates = project.get_locked_repository().candidates
     assert locked_candidates["requests"].version == "2.20.0"
     assert locked_candidates["chardet"].version == "3.0.5"
     assert locked_candidates["pytz"].version == "2019.3"
@@ -235,7 +303,7 @@ def test_update_package_with_extras(project, repository, working_set, pdm):
     repository.add_dependencies("foo", "0.2", foo_deps)
     pdm(["update"], obj=project, strict=True)
     assert working_set["foo"].version == "0.2"
-    assert project.locked_repository.all_candidates["foo"].version == "0.2"
+    assert project.get_locked_repository().candidates["foo"].version == "0.2"
 
 
 def test_update_groups_in_lockfile(project, working_set, pdm, repository):
@@ -245,13 +313,22 @@ def test_update_groups_in_lockfile(project, working_set, pdm, repository):
     assert project.lockfile.groups == ["default", "extra"]
     repository.add_candidate("foo", "0.2")
     pdm(["update"], obj=project, strict=True)
-    assert project.locked_repository.all_candidates["foo"].version == "0.2"
+    assert project.get_locked_repository().candidates["foo"].version == "0.2"
     assert working_set["foo"].version == "0.2"
 
 
 def test_update_group_not_in_lockfile(project, working_set, pdm):
     pdm(["add", "requests"], obj=project, strict=True)
-    project.add_dependencies({"pytz": parse_requirement("pytz")}, to_group="extra")
+    project.add_dependencies(["pytz"], to_group="extra")
     result = pdm(["update", "--group", "extra"], obj=project)
     assert result.exit_code != 0
     assert "Requested groups not in lockfile: extra" in result.stderr
+
+
+@pytest.mark.usefixtures("working_set")
+def test_update_dependency_group_with_include(project, pdm):
+    from pdm.formats.base import make_array
+
+    project.pyproject.dependency_groups.update({"tz": ["pytz"], "web": make_array([{"include-group": "tz"}])})
+    project.pyproject.write()
+    pdm(["update", "-u"], obj=project, strict=True)
