@@ -1,4 +1,5 @@
 import os
+import platform
 import re
 import shutil
 import sys
@@ -133,7 +134,25 @@ def test_venv_activate(pdm, mocker, project):
         assert result.output.startswith("conda activate")
     else:
         assert result.output.strip("'\"\n").endswith("activate")
-        assert result.output.startswith("source")
+        if platform.system() == "Windows":
+            assert not result.output.startswith("source")
+            assert not result.output.startswith("'")
+        else:
+            assert result.output.startswith("source")
+
+
+@pytest.mark.usefixtures("venv_backends")
+@pytest.mark.skipif(platform.system() == "Windows", reason="UNIX only")
+def test_venv_activate_tcsh(pdm, mocker, project):
+    project.project_config["venv.in_project"] = False
+    result = pdm(["venv", "create"], obj=project)
+    assert result.exit_code == 0, result.stderr
+    venv_path = re.match(r"Virtualenv (.+) is created successfully", result.output).group(1)
+    key = os.path.basename(venv_path)[len(get_venv_prefix(project)) :]
+
+    mocker.patch("shellingham.detect_shell", return_value=("tcsh", None))
+    result = pdm(["venv", "activate", key], obj=project)
+    assert result.output.startswith("source") and result.output.strip("'\"\n").endswith("activate.csh")
 
 
 @pytest.mark.usefixtures("venv_backends")
@@ -186,7 +205,11 @@ def test_venv_activate_no_shell(pdm, mocker, project):
         assert result.output.startswith("conda activate")
     else:
         assert result.output.strip("'\"\n").endswith("activate")
-        assert result.output.startswith("source")
+        if platform.system() == "Windows":
+            assert not result.output.startswith("source")
+            assert not result.output.startswith("'")
+        else:
+            assert result.output.startswith("source")
 
 
 @pytest.mark.usefixtures("fake_create")
@@ -199,7 +222,7 @@ def test_venv_auto_create(pdm, mocker, project, keep_pypackages):
     else:
         shutil.rmtree(project.root / "__pypackages__", ignore_errors=True)
     project.project_config["python.use_venv"] = True
-    pdm(["install"], obj=project)
+    pdm(["install", "--no-self"], obj=project)
     if keep_pypackages:
         creator.assert_not_called()
     else:
@@ -285,8 +308,9 @@ def test_venv_backend_create(project, mocker, with_pip):
 
 
 def test_conda_backend_create(project, mocker, with_pip):
-    backend = backends.CondaBackend(project, "3.8")
-    assert backend.ident == "3.8"
+    assert project.python
+    backend = backends.CondaBackend(project, "3.9")
+    assert backend.ident == "3.9"
     mock_call = mocker.patch("subprocess.check_call")
     location = backend.create(with_pip=with_pip)
     pip_args = ["pip"] if with_pip else []
@@ -297,7 +321,7 @@ def test_conda_backend_create(project, mocker, with_pip):
             "--yes",
             "--prefix",
             str(location),
-            "python=3.8",
+            "python=3.9",
             *pip_args,
         ],
         stdout=ANY,
