@@ -2,8 +2,6 @@ from __future__ import annotations
 
 import os
 import shutil
-import sys
-import unittest.mock as mock
 from pathlib import Path
 from typing import TYPE_CHECKING, Iterable
 from urllib.parse import unquote, urlparse
@@ -11,7 +9,8 @@ from urllib.parse import unquote, urlparse
 import pytest
 from unearth.vcs import Git, vcs_support
 
-from pdm._types import RepositoryConfig
+from pdm.models.auth import keyring
+from pdm.project import Project
 from tests import FIXTURES
 
 if TYPE_CHECKING:
@@ -26,10 +25,15 @@ pytest_plugins = [
 
 
 @pytest.fixture
-def index() -> dict[str, str]:
+def index() -> dict[str, bytes]:
     return {}
 
 
+@pytest.fixture(scope="session", autouse=True)
+def disable_keyring():
+    keyring.enabled = False
+
+
 @pytest.fixture
 def pypi_indexes(index) -> IndexesDefinition:
     return {
@@ -39,6 +43,7 @@ def pypi_indexes(index) -> IndexesDefinition:
         "https://my.pypi.org/": (
             {
                 "/simple": FIXTURES / "index",
+                "/json": FIXTURES / "json",
             },
             index,
             True,
@@ -46,6 +51,11 @@ def pypi_indexes(index) -> IndexesDefinition:
     }
 
 
+def pytest_runtest_setup(item):
+    if "uv" in item.keywords and not shutil.which("uv"):
+        pytest.skip("uv command not found")
+
+
 class MockGit(Git):
     def fetch_new(self, location, url, rev, args):
         path = os.path.splitext(os.path.basename(unquote(urlparse(str(url)).path)))[0]
@@ -99,13 +109,23 @@ def copytree(src: Path, dst: Path) -> None:
 
 
 @pytest.fixture()
-def fixture_project(project_no_init):
+def fixture_project(project_no_init: Project, request: pytest.FixtureRequest, local_finder_artifacts: Path):
     """Initialize a project from a fixture project"""
 
     def func(project_name):
         source = FIXTURES / "projects" / project_name
         copytree(source, project_no_init.root)
         project_no_init.pyproject.reload()
+        if "local_finder" in request.fixturenames:
+            project_no_init.pyproject.settings["source"] = [
+                {
+                    "type": "find_links",
+                    "verify_ssl": False,
+                    "url": local_finder_artifacts.as_uri(),
+                    "name": "pypi",
+                }
+            ]
+            project_no_init.pyproject.write()
         project_no_init.environment = None
         return project_no_init
 
@@ -126,55 +146,3 @@ def is_editable(request):
 @pytest.fixture(params=[False, True])
 def dev_option(request) -> Iterable[str]:
     return ("--dev",) if request.param else ()
-
-
-class _RepositoryConfigFactory:
-    @classmethod
-    def get_repository_config(cls, **kwargs):
-        return mock.create_autospec(RepositoryConfig, instance=False, **kwargs)
-
-
-@pytest.fixture(scope="function")
-def repository_configs(request):
-    return [
-        _RepositoryConfigFactory.get_repository_config(**config_params)
-        for config_params in request.param["config_params"]
-    ]
-
-
-class _PathFactory:
-    is_pre_py312 = sys.version_info < (3, 12)
-
-    @classmethod
-    def get_path(cls, **kwargs):
-        path = Path(kwargs["pathstr"])
-        kwargs.pop("pathstr")
-
-        return mock.create_autospec(path, instance=False, **kwargs)
-
-    @classmethod
-    def get_py312_compatible_path(cls, **kwargs):
-        """
-        Equality checking of``pathlib.Path`` objects is different in
-        pre-Python 3.12 and Python 3.12 versions.
-
-        This factory method returns a mock ``pathlib.Path`` object from a
-        real path, which will satisfy an ``__eq__`` test on pre-Python 3.12
-        and Python 3.12 versions.
-        """
-        path = Path(kwargs["pathstr"])
-
-        if cls.is_pre_py312:
-            return mock.create_autospec(path, instance=True, _cparts=path._cparts, _flavour=path._flavour)
-        else:
-            return mock.create_autospec(path, instance=True, _str_normcase=path._str_normcase, _flavour=path._flavour)
-
-
-@pytest.fixture(scope="function")
-def paths(request):
-    return [_PathFactory.get_path(**path_params) for path_params in request.param["path_params"]]
-
-
-@pytest.fixture(scope="function")
-def py312_compatible_paths(request):
-    return [_PathFactory.get_py312_compatible_path(**path_params) for path_params in request.param["path_params"]]
