@@ -9,21 +9,15 @@ from pdm.models.specifiers import PySpecSet
     [
         (">=3.6", ">=3.6"),
         ("<3.8", "<3.8"),
-        ("~=2.7.0", ">=2.7,<2.8"),
+        ("~=2.7.0", "~=2.7.0"),
         ("", ""),
-        (">=3.6,<3.8", ">=3.6,<3.8"),
-        (">3.6", ">=3.6.1"),
-        ("<=3.7", "<3.7.1"),
-        ("<3.3,!=3.4.*,!=3.5.*", "<3.3"),
-        (">=3.6,!=3.4.*", ">=3.6"),
-        (">=3.6,!=3.6.*", ">=3.7"),
-        (">=3.6,<3.8,!=3.8.*", ">=3.6,<3.8"),
-        (">=2.7,<3.2,!=3.0.*,!=3.1.*", ">=2.7,<3.0"),
-        ("!=3.0.*,!=3.0.2", "!=3.0.*"),
-        (">=3.4.*", ">=3.4"),
-        (">3.4.*", ">=3.4"),
-        ("<=3.4.*", "<3.4"),
-        ("<3.4.*", "<3.4"),
+        (">=3.6,<3.8", "<3.8,>=3.6"),
+        (">3.6", ">3.6"),
+        ("<=3.7", "<=3.7"),
+        (">=3.4.*", ">=3.4.0"),
+        (">3.4.*", ">=3.4.0"),
+        ("<=3.4.*", "<3.4.0"),
+        ("<3.4.*", "<3.4.0"),
         (">=3.0+g1234", ">=3.0"),
         ("<3.0+g1234", "<3.0"),
         ("<3.10.0a6", "<3.10.0a6"),
@@ -39,18 +33,18 @@ def test_normalize_pyspec(original, normalized):
     "left,right,result",
     [
         (">=3.6", ">=3.0", ">=3.6"),
-        (">=3.6", "<3.8", ">=3.6,<3.8"),
+        (">=3.6", "<3.8", "<3.8,>=3.6"),
         ("", ">=3.6", ">=3.6"),
-        (">=3.6", "<3.2", "impossible"),
-        (">=2.7,!=3.0.*", "!=3.1.*", ">=2.7,!=3.0.*,!=3.1.*"),
+        (">=3.6", "<3.2", "<empty>"),
+        (">=2.7,!=3.0.*", "!=3.1.*", "!=3.0.*,!=3.1.*,>=2.7"),
         (">=3.11.0a2", "<3.11.0b", ">=3.11.0a2,<3.11.0b0"),
-        ("<3.11.0a2", ">3.11.0b", "impossible"),
+        ("<3.11.0a2", ">3.11.0b", "<empty>"),
     ],
 )
 def test_pyspec_and_op(left, right, result):
     left = PySpecSet(left)
     right = PySpecSet(right)
-    assert str(left & right) == result
+    assert left & right == PySpecSet(result)
 
 
 @pytest.mark.parametrize(
@@ -59,10 +53,10 @@ def test_pyspec_and_op(left, right, result):
         (">=3.6", ">=3.0", ">=3.0"),
         ("", ">=3.6", ""),
         (">=3.6", "<3.7", ""),
-        (">=3.6,<3.8", ">=3.4,<3.7", ">=3.4,<3.8"),
-        ("~=2.7", ">=3.6", ">=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*,!=3.5.*"),
+        (">=3.6,<3.8", ">=3.4,<3.7", "<3.8,>=3.4"),
+        ("~=2.7", ">=3.6", "!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*,!=3.4.*,!=3.5.*,>=2.7"),
         ("<2.7.15", ">=3.0", "!=2.7.15,!=2.7.16,!=2.7.17,!=2.7.18"),
-        (">3.11.0a2", ">3.11.0b", ">=3.11.0a3"),
+        (">3.11.0a2", ">3.11.0b", ">3.11.0a2"),
     ],
 )
 def test_pyspec_or_op(left, right, result):
@@ -74,12 +68,9 @@ def test_pyspec_or_op(left, right, result):
 def test_impossible_pyspec():
     spec = PySpecSet(">=3.6,<3.4")
     a = PySpecSet(">=2.7")
-    assert spec.is_impossible
-    assert (spec & a).is_impossible
+    assert spec.is_empty()
+    assert (spec & a).is_empty()
     assert spec | a == a
-    spec_copy = spec.copy()
-    assert spec_copy.is_impossible
-    assert str(spec_copy) == "impossible"
 
 
 @pytest.mark.filterwarnings("ignore::FutureWarning")
