@@ -1,4 +1,3 @@
-import os
 import pathlib
 import sys
 import unittest.mock as mock
@@ -6,9 +5,9 @@ from pathlib import Path
 
 import pytest
 import tomlkit
-from packaging.version import Version
 
 from pdm import utils
+from pdm._types import RepositoryConfig
 from pdm.cli import utils as cli_utils
 from pdm.cli.filters import GroupSelection
 from pdm.exceptions import PdmUsageError, PDMWarning
@@ -41,27 +40,17 @@ def test_create_tracked_tempdir(mock_tempfile_mkdtemp, mock_os_makedirs, mock_at
     assert received_dirname == dirname
 
 
-@pytest.mark.parametrize(
-    "repository_configs",
-    [
+def test_get_trusted_hosts(mocker):
+    repository_configs = [
         {
-            "config_params": [
-                {
-                    "url": "https://pypi.org",
-                    "verify_ssl": False,
-                },
-                {"url": "https://untrusted.pypi.index", "verify_ssl": True},
-                {"url": "https://user:password@trusted1.pypi.index", "verify_ssl": False},
-                {"url": "https://user:password@trusted2.pypi.index", "verify_ssl": False},
-            ]
-        }
-    ],
-    indirect=[
-        "repository_configs",
-    ],
-)
-def test_get_trusted_hosts(repository_configs):
-    sources = repository_configs
+            "url": "https://pypi.org",
+            "verify_ssl": False,
+        },
+        {"url": "https://untrusted.pypi.index", "verify_ssl": True},
+        {"url": "https://user:password@trusted1.pypi.index", "verify_ssl": False},
+        {"url": "https://user:password@trusted2.pypi.index", "verify_ssl": False},
+    ]
+    sources = [mocker.create_autospec(RepositoryConfig, instance=False, **config) for config in repository_configs]
     expected = [
         "pypi.org",
         "trusted1.pypi.index",
@@ -166,7 +155,7 @@ def test_add_ssh_scheme_to_git_uri(given, expected):
 
 class TestUrlToPath:
     def test_non_file_url(self):
-        with pytest.raises(AssertionError):
+        with pytest.raises(ValueError):
             utils.url_to_path("not_a_file_scheme://netloc/path")
 
     @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows test")
@@ -183,27 +172,6 @@ class TestUrlToPath:
         assert utils.url_to_path("file://localhost/local/file/path") == "\\local\\file\\path"
 
 
-# Only testing POSIX-style paths here
-@pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
-@pytest.mark.parametrize(
-    "given, expected",
-    [
-        ("/path/to/my/pdm", "file:///path/to/my/pdm"),
-        ("/abs/path/to/my/pdm", "file:///abs/path/to/my/pdm"),
-        ("/path/to/my/pdm/pyproject.toml", "file:///path/to/my/pdm/pyproject.toml"),
-        ("../path/to/my/pdm/pyproject.toml", "file:///abs/path/to/my/pdm/pyproject.toml"),
-    ],
-)
-def test_path_to_url(given, expected):
-    if os.path.isabs(given):
-        assert utils.path_to_url(given) == expected
-    else:
-        abs_given = "abs" + str(given).replace("..", "")
-
-        with mock.patch("pdm.utils.os.path.abspath", return_value=abs_given):
-            assert utils.path_to_url(given) == expected
-
-
 @pytest.mark.parametrize(
     "given,expected",
     [
@@ -227,11 +195,11 @@ def test_expand_env_vars(given, expected, monkeypatch):
         ("https://example.org/path?arg=1", "https://example.org/path?arg=1"),
         (
             "https://${FOO}@example.org/path?arg=1",
-            "https://hello@example.org/path?arg=1",
+            "https://token%3Aoidc%2F1@example.org/path?arg=1",
         ),
         (
             "https://${FOO}:${BAR}@example.org/path?arg=1",
-            "https://hello:wo%3Arld@example.org/path?arg=1",
+            "https://token%3Aoidc%2F1:p%40ssword@example.org/path?arg=1",
         ),
         (
             "https://${FOOBAR}@example.org/path?arg=1",
@@ -240,8 +208,8 @@ def test_expand_env_vars(given, expected, monkeypatch):
     ],
 )
 def test_expand_env_vars_in_auth(given, expected, monkeypatch):
-    monkeypatch.setenv("FOO", "hello")
-    monkeypatch.setenv("BAR", "wo:rld")
+    monkeypatch.setenv("FOO", "token:oidc/1")
+    monkeypatch.setenv("BAR", "p@ssword")
     assert utils.expand_env_vars_in_auth(given) == expected
 
 
@@ -284,92 +252,59 @@ def test_is_path_relative_to(given, expected):
 
 
 class TestGetVenvLikePrefix:
-    @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
-    @pytest.mark.parametrize(
-        "py312_compatible_paths",
-        [
-            {"path_params": [{"pathstr": "/my/conda/bin"}]},
-        ],
-        indirect=["py312_compatible_paths"],
-    )
-    @mock.patch("pdm.utils.Path")
-    def test_conda_env_with_conda_meta_in_bin(self, path_patch, py312_compatible_paths):
-        path = Path("/my/conda/bin/python3")
-        interpreter_bin_path = py312_compatible_paths[0]
-        interpreter_bin_path.joinpath.return_value.exists.return_value = True
-        path_patch.return_value.parent = interpreter_bin_path
-        with path_patch:
-            received = utils.get_venv_like_prefix(str(path))
-            expected = interpreter_bin_path, True
-            assert received == expected
+    def test_conda_env_with_conda_meta_in_bin(self, tmp_path: Path):
+        path = tmp_path / "conda/bin/python3"
+        path_parent = path.parent
+        path_parent.mkdir(parents=True)
+        path.touch()
+        path_parent.joinpath("conda-meta").mkdir()
+        received = utils.get_venv_like_prefix(path)
+        expected = path_parent, True
+        assert received == expected
 
-    @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
-    @pytest.mark.parametrize(
-        "py312_compatible_paths",
-        [
-            {"path_params": [{"pathstr": "/my/local/py/bin"}, {"pathstr": "/my/local/py"}]},
-        ],
-        indirect=["py312_compatible_paths"],
-    )
-    @mock.patch("pdm.utils.Path")
-    def test_py_env_with_pyvenv_cfg(self, path_patch, py312_compatible_paths):
-        path = Path("/my/local/py/bin/python3")
-        interpreter_bin_path = py312_compatible_paths[0]
-        interpreter_bin_parent_path = py312_compatible_paths[1]
-        interpreter_bin_path.joinpath.return_value.exists.return_value = False
-        interpreter_bin_parent_path.joinpath.return_value.exists.return_value = True
-        path_patch.return_value.parent = interpreter_bin_path
-        path_patch.return_value.parent.parent = interpreter_bin_parent_path
-        with path_patch:
-            received = utils.get_venv_like_prefix(str(path))
-            expected = interpreter_bin_parent_path, False
-            assert received == expected
+    def test_py_env_with_pyvenv_cfg(self, tmp_path: Path):
+        path = tmp_path / "venv/bin/python3"
+        bin_path = path.parent
+        venv_path = path.parent.parent
+        bin_path.mkdir(parents=True)
+        path.touch()
+        venv_path.joinpath("pyvenv.cfg").touch()
 
-    @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
-    @pytest.mark.parametrize(
-        "py312_compatible_paths",
-        [
-            {"path_params": [{"pathstr": "/my/conda/bin/"}, {"pathstr": "/my/conda"}]},
-        ],
-        indirect=["py312_compatible_paths"],
-    )
-    @mock.patch("pdm.utils.Path")
-    def test_conda_env_with_conda_meta(self, path_patch, py312_compatible_paths):
-        path = Path("/my/conda/bin/python3")
-        interpreter_bin_path = py312_compatible_paths[0]
-        interpreter_bin_parent_path = py312_compatible_paths[1]
-        interpreter_bin_path.joinpath.return_value.exists.return_value = False
-        interpreter_bin_parent_path.joinpath.return_value.exists.side_effect = [False, True]
-        path_patch.return_value.parent = interpreter_bin_path
-        path_patch.return_value.parent.parent = interpreter_bin_parent_path
-        with path_patch:
-            received = utils.get_venv_like_prefix(str(path))
-            expected = interpreter_bin_parent_path, True
-            assert received == expected
+        received = utils.get_venv_like_prefix(str(path))
+        expected = venv_path, False
+        assert received == expected
 
-    @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
-    def test_virtual_env(self):
+    def test_conda_env_with_conda_meta(self, tmp_path: Path):
+        path = tmp_path / "conda/bin/python3"
+        interpreter_bin_path = path.parent
+        interpreter_bin_parent_path = interpreter_bin_path.parent
+        interpreter_bin_path.mkdir(parents=True)
+        path.touch()
+        interpreter_bin_parent_path.joinpath("conda-meta").mkdir()
+
+        received = utils.get_venv_like_prefix(str(path))
+        expected = interpreter_bin_parent_path, True
+        assert received == expected
+
+    def test_virtual_env(self, monkeypatch):
         path = Path("/my/venv")
         expected = path, False
-        with mock.patch.dict("pdm.utils.os.environ", {"VIRTUAL_ENV": str(path)}, clear=True):
-            received = utils.get_venv_like_prefix(path.joinpath("bin", "python3"))
-            assert received == expected
+        monkeypatch.setenv("VIRTUAL_ENV", str(path))
+        received = utils.get_venv_like_prefix(path.joinpath("bin", "python3"))
+        assert received == expected
 
-    @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
-    def test_conda_virtual_env(self):
+    def test_conda_virtual_env(self, monkeypatch):
         path = Path("/my/conda/venv")
         expected = path, True
-        with mock.patch.dict("pdm.utils.os.environ", {"CONDA_PREFIX": str(path)}, clear=True):
-            received = utils.get_venv_like_prefix(path.joinpath("bin", "python3"))
-            assert received == expected
+        monkeypatch.setenv("CONDA_PREFIX", str(path))
+        received = utils.get_venv_like_prefix(path.joinpath("bin", "python3"))
+        assert received == expected
 
-    @pytest.mark.skipif(sys.platform.startswith("win"), reason="Non-Windows tests")
     def test_no_virtual_env(self):
         path = Path("/not/a/venv/bin/python3")
         expected = None, False
-        with mock.patch.dict("pdm.utils.os.environ", {"VIRTUAL_ENV": "", "CONDA_PREFIX": ""}, clear=True):
-            received = utils.get_venv_like_prefix(str(path))
-            assert received == expected
+        received = utils.get_venv_like_prefix(str(path))
+        assert received == expected
 
 
 def compare_python_paths(path1, path2):
@@ -474,7 +409,7 @@ def setup_dependencies(project):
             "optional-dependencies": {"web": ["flask"], "auth": ["passlib"]},
         }
     )
-    project.pyproject.settings.update({"dev-dependencies": {"test": ["pytest"], "doc": ["mkdocs"]}})
+    project.pyproject.dependency_groups.update({"test": ["pytest"], "doc": ["mkdocs"]})
     project.pyproject.write()
 
 
@@ -509,6 +444,20 @@ def test_dependency_group_selection(project, args, golden):
     assert sorted(golden) == sorted(selection)
 
 
+@pytest.mark.parametrize(
+    "args,golden",
+    [
+        ({"groups": [":all"], "excluded_groups": ["web"]}, ["default", "auth", "doc", "test"]),
+        ({"groups": [":all"], "excluded_groups": ["web", "auth"]}, ["default", "doc", "test"]),
+        ({"groups": [":all"], "excluded_groups": ["default", "test"]}, ["auth", "doc", "web"]),
+    ],
+)
+def test_exclude_optional_groups_from_all(project, args, golden):
+    setup_dependencies(project)
+    selection = GroupSelection(project, **args)
+    assert golden == list(selection)
+
+
 def test_prod_should_not_be_with_dev(project):
     setup_dependencies(project)
     selection = GroupSelection(project, default=True, dev=False, groups=["test"])
@@ -527,8 +476,8 @@ def test_deprecation_warning():
 
 
 def test_comparable_version():
-    assert utils.comparable_version("1.2.3") == Version("1.2.3")
-    assert utils.comparable_version("1.2.3a1+local1") == Version("1.2.3a1")
+    assert utils.comparable_version("1.2.3") == utils.parse_version("1.2.3")
+    assert utils.comparable_version("1.2.3a1+local1") == utils.parse_version("1.2.3a1")
 
 
 @pytest.mark.parametrize("name", ["foO", "f", "3d", "f3", "333", "foo.bar", "foo-bar", "foo_bar", "foo-_.bar"])
